from typing import List, Union

import httpx
from typing_extensions import Literal

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH as path
from slark.types.spreadsheets import data


class AsyncData(AsyncAPIResource):
    async def prepend_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.PrependDataResponse:
        return await self._post(
            path.spreadsheets.data.prepend_data.format(spreadsheetToken=spreadsheet_token),
            body=data.PrependDataBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.PrependDataResponse,
            options={"timeout": timeout},
        )

    async def append_data(
        self,
        spreadsheet_token: str,
        *,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.AppendDataResponse:
        """在电子表格工作表的指定范围中，在空白位置中追加数据。例如，若指定范围参数 range 为 6e5ed3!A1:B2，该接口将会依次寻找 A1、A2、A3...单元格，在找到的第一个空白位置中写入数据。

            使用限制
            单次写入范围不可超过 5,000 行、100 列。
            每个单元格不超过 50,000 字符，由于服务端会增加控制字符，因此推荐每个单元格不超过 40,000 字符。


        Args:
            spreadsheet_token (str): 电子表格的 token
            range (str): 追加数据的范围。格式为 <sheetId>!<开始位置>:<结束位置>
            values (List[List[data.CellTypes]]): 追加的数据

            insertDataOption (Literal[&quot;OVERWRITE&quot;, &quot;INSERT_ROWS&quot;], optional):
            指定追加数据的方式，默认值为 OVERWRITE，即若空行数量小于追加数据的行数，则会覆盖已有数据。可选值：

            OVERWRITE：若空行的数量小于追加数据的行数，则会覆盖已有数据
            INSERT_ROWS：插入足够数量的行后再进行数据追加. Defaults to "OVERWRITE".

            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            AppendDataResponse: AppendDataResponse
        """
        return await self._post(
            path.spreadsheets.data.append_data.format(spreadsheetToken=spreadsheet_token),
            body=data.AppendDataBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.AppendDataResponse,
            options={
                "timeout": timeout,
                "params": data.AppendDataParams(insertDataOption=insertDataOption).model_dump(),
            },
        )

    async def write_single_range_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.WriteSingleRangeResponse:
        """向电子表格某个工作表的单个指定范围中写入数据。若指定范围已内有数据，将被新写入的数据覆盖。

            使用限制
            单次写入数据不得超过 5000 行、100列。
            每个单元格不超过 50,000 字符，由于服务端会增加控制字符，因此推荐每个单元格不超过 40,000 字符。

        Args:
            spreadsheet_token (str): _description_
            range (str): 写入数据的范围。格式为 <sheetId>!<开始位置>:<结束位置>。其中：

            sheetId 为工作表 ID，通过获取工作表 获取。

            <开始位置>:<结束位置> 为工作表中单元格的范围，数字表示行索引，字母表示列索引。如 A2:B2 表示该工作表第 2 行的 A 列到 B 列。range支持四种写法，详情参考电子表格概述。

            注意：range 所指定的范围需要大于等于写入的数据所占用的范围。
            values (List[List[data.CellTypes]]): 写入的数据。如要写入公式、超链接、email、@人等，可参考电子表格支持写入的数据类型。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            WriteSingleRangeResponse: WriteSingleRangeResponse
        """
        return await self._put(
            path.spreadsheets.data.write_single_range_data.format(
                spreadsheetToken=spreadsheet_token
            ),
            body=data.WriteSingleRangeBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.WriteSingleRangeResponse,
            options={"timeout": timeout},
        )

    async def write_multi_range_data(
        self,
        spreadsheet_token: str,
        *,
        value_ranges: List[data.WriteDataValueRange],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.WriteMultiRangeResponse:
        return await self._post(
            path.spreadsheets.data.write_multi_range_data.format(
                spreadsheetToken=spreadsheet_token
            ),
            body=data.WriteMultiRangeBody(valueRanges=value_ranges).model_dump(),
            cast_to=data.WriteMultiRangeResponse,
            options={"timeout": timeout},
        )

    async def read_single_range_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.ReadSingleRangeResponse:
        """读取电子表格中单个指定范围的数据。

        使用限制
        该接口返回数据的最大限制为 10 MB。
        该接口不支持获取跨表引用和数组公式的计算结果。

        Args:
            spreadsheet_token (str): 电子表格的 token
            range (str): 查询范围。格式为 `<sheetId>!<开始位置>:<结束位置>`

            valueRenderOption (Literal[ &quot;ToString&quot;, &quot;Formula&quot;, &quot;FormattedValue&quot;, &quot;UnformattedValue&quot; ], None, optional): 指定单元格数据的格式。可选值如下所示。当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。
            ToString：返回纯文本的值（数值类型除外）
            Formula：单元格中含有公式时，返回公式本身
            FormattedValue：计算并格式化单元格
            UnformattedValue：计算但不对单元格进行格式化. Defaults to None.

            dateTimeRenderOption (Literal[&quot;FormattedString&quot;], None, optional):
            指定数据类型为日期、时间、或时间日期的单元格数据的格式。
            若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；小数部分为该时间占 24 小时的份额。例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。
            可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。. Defaults to None.

            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;], None, optional):
            当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。默认为 lark_id，建议选择 open_id 或 union_id. Defaults to None.

            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            _type_: _description_
        """
        return await self._get(
            path.spreadsheets.data.read_single_range_data.format(
                spreadsheetToken=spreadsheet_token, range=range
            ),
            cast_to=data.ReadSingleRangeResponse,
            options={
                "params": data.ReadSingleRangeParams(
                    valueRenderOption=valueRenderOption,
                    dateTimeRenderOption=dateTimeRenderOption,
                    user_id_type=user_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )

    async def read_multi_range_data(
        self,
        spreadsheet_token: str,
        *,
        ranges: str,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.ReadMultiRangeResponse:
        return await self._get(
            path.spreadsheets.data.read_multi_range_data.format(spreadsheetToken=spreadsheet_token),
            cast_to=data.ReadMultiRangeResponse,
            options={
                "params": data.ReadMultiRangeParams(
                    ranges=ranges,
                    valueRenderOption=valueRenderOption,
                    dateTimeRenderOption=dateTimeRenderOption,
                    user_id_type=user_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )


class Data(APIResource):
    def prepend_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.PrependDataResponse:
        return self._post(
            path.spreadsheets.data.prepend_data.format(spreadsheetToken=spreadsheet_token),
            body=data.PrependDataBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.PrependDataResponse,
            options={"timeout": timeout},
        )

    def append_data(
        self,
        spreadsheet_token: str,
        *,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.AppendDataResponse:
        """在电子表格工作表的指定范围中，在空白位置中追加数据。例如，若指定范围参数 range 为 6e5ed3!A1:B2，该接口将会依次寻找 A1、A2、A3...单元格，在找到的第一个空白位置中写入数据。

            使用限制
            单次写入范围不可超过 5,000 行、100 列。
            每个单元格不超过 50,000 字符，由于服务端会增加控制字符，因此推荐每个单元格不超过 40,000 字符。


        Args:
            spreadsheet_token (str): 电子表格的 token
            range (str): 追加数据的范围。格式为 <sheetId>!<开始位置>:<结束位置>
            values (List[List[data.CellTypes]]): 追加的数据

            insertDataOption (Literal[&quot;OVERWRITE&quot;, &quot;INSERT_ROWS&quot;], optional):
            指定追加数据的方式，默认值为 OVERWRITE，即若空行数量小于追加数据的行数，则会覆盖已有数据。可选值：

            OVERWRITE：若空行的数量小于追加数据的行数，则会覆盖已有数据
            INSERT_ROWS：插入足够数量的行后再进行数据追加. Defaults to "OVERWRITE".

            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            AppendDataResponse: AppendDataResponse
        """
        return self._post(
            path.spreadsheets.data.append_data.format(spreadsheetToken=spreadsheet_token),
            body=data.AppendDataBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.AppendDataResponse,
            options={
                "timeout": timeout,
                "params": data.AppendDataParams(insertDataOption=insertDataOption).model_dump(),
            },
        )

    def write_single_range_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        values: List[List[data.CellTypes]],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.WriteSingleRangeResponse:
        """向电子表格某个工作表的单个指定范围中写入数据。若指定范围已内有数据，将被新写入的数据覆盖。

            使用限制
            单次写入数据不得超过 5000 行、100列。
            每个单元格不超过 50,000 字符，由于服务端会增加控制字符，因此推荐每个单元格不超过 40,000 字符。

        Args:
            spreadsheet_token (str): _description_
            range (str): 写入数据的范围。格式为 <sheetId>!<开始位置>:<结束位置>。其中：

            sheetId 为工作表 ID，通过获取工作表 获取。

            <开始位置>:<结束位置> 为工作表中单元格的范围，数字表示行索引，字母表示列索引。如 A2:B2 表示该工作表第 2 行的 A 列到 B 列。range支持四种写法，详情参考电子表格概述。

            注意：range 所指定的范围需要大于等于写入的数据所占用的范围。
            values (List[List[data.CellTypes]]): 写入的数据。如要写入公式、超链接、email、@人等，可参考电子表格支持写入的数据类型。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            WriteSingleRangeResponse: WriteSingleRangeResponse
        """
        return self._put(
            path.spreadsheets.data.write_single_range_data.format(
                spreadsheetToken=spreadsheet_token
            ),
            body=data.WriteSingleRangeBody(
                valueRange=data.WriteDataValueRange(range=range, values=values)
            ).model_dump(),
            cast_to=data.WriteSingleRangeResponse,
            options={"timeout": timeout},
        )

    def write_multi_range_data(
        self,
        spreadsheet_token: str,
        *,
        value_ranges: List[data.WriteDataValueRange],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.WriteMultiRangeResponse:
        return self._post(
            path.spreadsheets.data.write_multi_range_data.format(
                spreadsheetToken=spreadsheet_token
            ),
            body=data.WriteMultiRangeBody(valueRanges=value_ranges).model_dump(),
            cast_to=data.WriteMultiRangeResponse,
            options={"timeout": timeout},
        )

    def read_single_range_data(
        self,
        spreadsheet_token: str,
        *,
        range: str,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.ReadSingleRangeResponse:
        """读取电子表格中单个指定范围的数据。

        使用限制
        该接口返回数据的最大限制为 10 MB。
        该接口不支持获取跨表引用和数组公式的计算结果。

        Args:
            spreadsheet_token (str): 电子表格的 token
            range (str): 查询范围。格式为 `<sheetId>!<开始位置>:<结束位置>`

            valueRenderOption (Literal[ &quot;ToString&quot;, &quot;Formula&quot;, &quot;FormattedValue&quot;, &quot;UnformattedValue&quot; ], None, optional): 指定单元格数据的格式。可选值如下所示。当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。
            ToString：返回纯文本的值（数值类型除外）
            Formula：单元格中含有公式时，返回公式本身
            FormattedValue：计算并格式化单元格
            UnformattedValue：计算但不对单元格进行格式化. Defaults to None.

            dateTimeRenderOption (Literal[&quot;FormattedString&quot;], None, optional):
            指定数据类型为日期、时间、或时间日期的单元格数据的格式。
            若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；小数部分为该时间占 24 小时的份额。例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。
            可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。. Defaults to None.

            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;], None, optional):
            当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。默认为 lark_id，建议选择 open_id 或 union_id. Defaults to None.

            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            _type_: _description_
        """
        return self._get(
            path.spreadsheets.data.read_single_range_data.format(
                spreadsheetToken=spreadsheet_token, range=range
            ),
            cast_to=data.ReadSingleRangeResponse,
            options={
                "params": data.ReadSingleRangeParams(
                    valueRenderOption=valueRenderOption,
                    dateTimeRenderOption=dateTimeRenderOption,
                    user_id_type=user_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )

    def read_multi_range_data(
        self,
        spreadsheet_token: str,
        *,
        ranges: str,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> data.ReadMultiRangeResponse:
        return self._get(
            path.spreadsheets.data.read_multi_range_data.format(spreadsheetToken=spreadsheet_token),
            cast_to=data.ReadMultiRangeResponse,
            options={
                "params": data.ReadMultiRangeParams(
                    ranges=ranges,
                    valueRenderOption=valueRenderOption,
                    dateTimeRenderOption=dateTimeRenderOption,
                    user_id_type=user_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )



import typing as t

from typing_extensions import Literal

from slark.types._common import BaseModel

MESSAGE_TYPE = Literal[
    "text",
    "post",
    "image",
    "file",
    "audio",
    "media",
    "sticker",
    "interactive",
    "share_card",
    "share_user",
]
"""text：文本
    post：富文本
    image：图片
    file：文件
    audio：语音
    media：视频
    sticker：表情包
    interactive：卡片
    share_card：分享群名片
    share_user：分享个人名片"""


class SendMessageParams(BaseModel):
    receive_id_type: Literal["open_id", "user_id", "email", "chat_id"]
    """用户 ID 类型

    示例值："open_id"

    可选值有：

    open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID
    union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？
    user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？
    email：以用户的真实邮箱来标识用户。
    chat_id：以群 ID 来标识群聊。了解更多：如何获取群 ID"""


class GetMessageParams(BaseModel):
    user_id_type: Literal["open_id", "union_id", "user_id"] = "open_id"


class SendMessageBody(BaseModel):
    receive_id: str
    """消息接收者的 ID，ID 类型与查询参数 receive_id_type 的取值一致。

    注意事项：

    给用户发送消息时，用户需要在机器人的可用范围内。例如，你需要给企业全员发送消息，则需要将应用的可用范围设置为全体员工。
    给群组发送消息时，机器人需要在该群组中，且在群组内拥有发言权限。
    如果消息接收者为用户，推荐使用用户的 open_id。"""

    msg_type: MESSAGE_TYPE
    content: str
    """消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。

    注意：
    JSON 字符串需进行转义。例如，换行符 \n 转义后为 \\n。
    文本消息请求体最大不能超过 150 KB。
    卡片消息、富文本消息请求体最大不能超过 30 KB。
    如果使用卡片模板（template_id）发送消息，实际大小也包含模板对应的卡片数据大小。
    如果消息中包含样式标签，会使实际消息体长度大于您输入的请求体长度。
    图片需要先上传图片，然后使用图片的 Key 发消息。
    音频、视频、文件需要先上传文件，然后使用文件的 Key 发消息。注意不能使用云文档上传素材接口返回的 file_token。
    了解不同类型的消息内容格式、使用限制，可参见发送消息内容。"""

    uuid: t.Union[str, None] = None
    """自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
        注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
        示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204" """


class ReplyMessageBody(BaseModel):
    content: str
    msg_type: MESSAGE_TYPE
    reply_in_thread: bool = False
    """是否以话题形式回复。取值为 true 时将以话题形式回复。\
        注意：如果要回复的消息已经是话题形式的消息，则默认以话题形式进行回复。"""

    uuid: t.Union[str, None] = None
    """自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
        注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
        示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204" """


class EditMessageBody(BaseModel):
    msg_type: Literal["text", "post"]
    content: str



from typing import Dict, Union

from slark.types.exceptions._base import LarkException


class LarkStatusCode:
    SUCCESS = 0
    REQUIRE_AUTHENTICATION = 10001
    RESOURCE_NOT_FOUND = 131005
    BAD_REQUEST = 10400
    INVALID_ACCESS_TOKEN = 20005
    PASSTIME_ACCESS_TOKEN = 20006
    INTERNAL_ERROR = 40003
    BAD_RESPONSE = 10002
    APITimeout = 10003
    APIConnectionError = 10004


class AuthenticationRequiredException(LarkException):
    def __init__(self, msg: str = "Require Authentication", context: Union[Dict, None] = None):
        super().__init__(LarkStatusCode.REQUIRE_AUTHENTICATION, msg, context)


class HttpStatusError(LarkException):
    def __init__(self, code: int, msg: str, context: Union[Dict, None] = None):
        super().__init__(code, msg, context)


class BadResponseError(LarkException):
    def __init__(self, msg: str = "Bad Response", context: Union[Dict, None] = None):
        super().__init__(LarkStatusCode.BAD_RESPONSE, msg, context)


class APITimeoutError(LarkException):
    def __init__(self, msg: str = "Timeout", context: Union[Dict, None] = None):
        super().__init__(LarkStatusCode.APITimeout, msg, context)


class APIConnectionError(LarkException):
    def __init__(self, msg: str = "API Connection Error", context: Union[Dict, None] = None):
        super().__init__(LarkStatusCode.APIConnectionError, msg, context)



from typing import Dict, Union

from typing_extensions import Literal

from .base import BaseModel


class OpenURLBehavior(BaseModel):
    type: str = "open_url"
    default_url: str
    android_url: Union[str, None] = None
    ios_url: Union[str, None] = None
    pc_url: Union[str, None] = None
    """可配置为 `lark://msgcard/unsupported_action` 声明当前端不允许跳转。"""


class CallbackBehavior(BaseModel):
    type: str = "callback"
    value: Union[str, Dict, None] = None


class FormActionBehavior(BaseModel):
    type: str = "form_action"
    value: Literal["submit", "reset"] = "submit"
    """表单事件类型。可取值：

    submit：提交整个表单
    reset：重置整个表单"""


BehaviorType = Union[OpenURLBehavior, CallbackBehavior, FormActionBehavior]



from typing import Dict, Union


class LarkException(Exception):
    def __init__(self, code: int, msg: str, context: Union[Dict, None] = None):
        self.code = code
        self.msg = msg
        self.context = context

    def __repr__(self) -> str:
        return f"{self.code}: {self.msg}"

    __str__ = __repr__



from typing import List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel

"""文档 https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/bitable-record-data-structure-overview"""


class MultiLineText(BaseModel):
    text: str
    type: Literal["text", "mention", "url"]
    mention_type: Union[Literal["User", "Doc", "Sheet", "Bitable", "Docx"], None] = None
    """type字段为mention，该字段有效，可选值有：User,Doc,Sheet,Bitable,Docx"""
    token: Union[str, None] = None
    """type字段为mention，该字段有效，不同mentionType该字段表示不同的含义
        mentionType	token含义
        User	用户id
        Doc	文档token
        Sheet	电子表格token
        Bitable	多维表格token
        Docx	文档token"""
    link: Union[str, None] = None
    """链接，type字段为url，该字段有效"""
    name: Union[str, None] = None
    """名称，type字段为url或mention，该字段有效"""


class User(BaseModel):
    id: str
    name: Union[str, None] = None
    """人员名字"""
    avatar_url: Union[str, None] = None
    """头像链接"""
    en_name: Union[str, None] = None
    """英文名字"""
    email: Union[str, None] = None
    """邮箱"""


class Group(BaseModel):
    name: Union[str, None] = None
    """群组名"""
    avatar_url: Union[str, None] = None
    """头像链接"""
    id: str
    """群组id"""


class HyperLink(BaseModel):
    text: str
    """文本名称"""
    link: str
    """超链接"""


class Attachment(BaseModel):
    file_token: str
    """附件token"""
    name: Union[str, None] = None
    """附件名称"""
    type: Union[str, None] = None
    """附件的 mime 类型, 如: image/png"""
    size: Union[int, None] = None
    """附件大小, 单位: 字节"""
    url: Union[str, None] = None
    """	附件url，下载请参考: 
        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/attachment#22daecaf"""
    tmp_url: Union[str, None] = None
    """生成附件临时下载链接的url，需access token鉴权"""


class LinkRecord(BaseModel):
    link_record_ids: List[str]
    """包含多个recordID字符串的数组"""


class Location(BaseModel):
    location: str
    """经纬度"""
    pname: str
    """省"""
    cityname: str
    """市"""
    adname: str
    """区"""
    address: str
    """详细地址"""
    name: str
    """地名"""
    full_address: str
    """完整地址"""


class Email(BaseModel):
    link: str
    """邮箱的 URL 链接。示例值：mailto:zhangmin@bytedance.com"""
    type: str = "url"
    """固定取值 "url"。"""
    text: str
    """用户邮箱。示例值："zhangmin@bytedance.com"。"""


FormulaType = Union[
    List[int],
    List[str],
    List[float],
    List[bool],
    List[MultiLineText],
    List[User],
    List[Group],
    List[HyperLink],
    List[Attachment],
    List[LinkRecord],
    List[Location],
    List[Email],
]


class Formula(BaseModel):
    type: int
    """value数据类型（相同字段类型用ui_type区分）：
    1：多行文本、条码
    2：数字、进度、货币、评分
    3：单选
    4：多选
    5：日期
    7：复选框
    11：人员
    13：电话号码
    15：超链接
    17：附件
    18：单向关联
    19：查找引用
    20：公式
    21：双向关联
    22：地理位置
    23：群组
    1001：创建时间
    1002：最后更新时间
    1003：创建人
    1004：修改人
    1005：自动编号"""
    value: List[FormulaType]


class Empty(BaseModel):
    pass


FieldValueType = Union[
    int,
    str,
    float,
    bool,
    User,
    None,
    List[str],
    List[MultiLineText],
    List[User],
    List[Group],
    HyperLink,
    List[Attachment],
    LinkRecord,
    Location,
    Formula,
    Email,
    Empty,
]



import enum
from typing import Union

from .._common import BaseModel
from .color import Color


class UDIconToken(enum.Enum):
    ADD_OUTLINED = "add_outlined"
    ADD_APP_OUTLINED = "add-app_outlined"
    ADD_BOLD_OUTLINED = "add-bold_outlined"
    ADD_MIDDLE_OUTLINED = "add-middle_outlined"
    ADMIN_OUTLINED = "admin_outlined"
    ALARM_CLOCK_OUTLINED = "alarm-clock_outlined"
    ALBUM_OUTLINED = "album_outlined"
    ALERTS_OFF_OUTLINED = "alerts-off_outlined"
    ALL_OUTLINED = "all_outlined"
    APP_DEFAULT_OUTLINED = "app-default_outlined"
    ARROW_OUTLINED = "arrow_outlined"
    ARROW_DOWN_OUTLINED = "arrow-down_outlined"
    ARROW_LEFT_OUTLINED = "arrow-left_outlined"
    ARROW_UP_OUTLINED = "arrow-up_outlined"
    BACK_TAB_OUTLINED = "back-tab_outlined"
    BAN_OUTLINED = "ban_outlined"
    BAN_CANCEL_OUTLINED = "ban-cancel_outlined"
    BELL_OUTLINED = "bell_outlined"
    CALENDAR_SLIDE_OUTLINED = "calendar-slide_outlined"
    CAMERA_OUTLINED = "camera_outlined"
    CANCEL_COLLECTION_OUTLINED = "cancel-collection_outlined"
    CHECK_OUTLINED = "check_outlined"
    CLOSE_OUTLINED = "close_outlined"
    CLOSE_BOLD_OUTLINED = "close-bold_outlined"
    CLOSE_SMALL_OUTLINED = "close-small_outlined"
    COLLAPSE_OUTLINED = "collapse_outlined"
    COLLECTION_OUTLINED = "collection_outlined"
    CUSTOMIZE_SIZE_OUTLINED = "customize-size_outlined"
    DELETE_OUTLINED = "delete_outlined"
    DELETE_APP_OUTLINED = "delete-app_outlined"
    DELETE_TRASH_OUTLINED = "delete-trash_outlined"
    DETAILS_OUTLINED = "details_outlined"
    DONE_OUTLINED = "done_outlined"
    DOWN_OUTLINED = "down_outlined"
    DOWN_BOLD_OUTLINED = "down-bold_outlined"
    DOWN_BOTTOM_OUTLINED = "down-bottom_outlined"
    DOWNLOAD_OUTLINED = "download_outlined"
    DOWN_RIGHT_OUTLINED = "down-right_outlined"
    DOWN_ROUND_OUTLINED = "down-round_outlined"
    EDIT_OUTLINED = "edit_outlined"
    EDIT_CONTINUE_OUTLINED = "edit-continue_outlined"
    ELLIPSE_OUTLINED = "ellipse_outlined"
    EXPAND_OUTLINED = "expand_outlined"
    EXPORT_OUTLINED = "export_outlined"
    FEED_READ_OUTLINED = "feed-read_outlined"
    FEED_UNREAD_OUTLINED = "feed-unread_outlined"
    FILTER_OUTLINED = "filter_outlined"
    FIND_APP_OUTLINED = "find-app_outlined"
    HISTORY_OUTLINED = "history_outlined"
    HISTORY_SEARCH_OUTLINED = "history-search_outlined"
    IMAGE_OUTLINED = "image_outlined"
    IMAGE_FAIL_OUTLINED = "image-fail_outlined"
    IMAGE_SEARCH_OUTLINED = "image-search_outlined"
    IMPORT_OUTLINED = "import_outlined"
    INDETERMINATE_OUTLINED = "indeterminate_outlined"
    INFO_OUTLINED = "info_outlined"
    INVISIBLE_OUTLINED = "invisible_outlined"
    LABEL_ADD_OUTLINED = "label-add_outlined"
    LABEL_CHANGE_OUTLINED = "label-change_outlined"
    LABEL_CUSTOM_OUTLINED = "label-custom_outlined"
    LABEL_REMOVE_OUTLINED = "label-remove_outlined"
    LEFT_OUTLINED = "left_outlined"
    LEFT_BOLD_OUTLINED = "left-bold_outlined"
    LEFT_SMALL_CCM_OUTLINED = "left-small-ccm_outlined"
    LIKE_OUTLINED = "like_outlined"
    LINE_FEED_OUTLINED = "line-feed_outlined"
    LINK_COPY_OUTLINED = "link-copy_outlined"
    LINK_LOCK_OUTLINED = "link-lock_outlined"
    LIST_CHECK_OUTLINED = "list-check_outlined"
    LIST_CHECK_BOLD_OUTLINED = "list-check-bold_outlined"
    LIST_FILTER_OUTLINED = "list-filter_outlined"
    LOADING_OUTLINED = "loading_outlined"
    LOGOUT_OUTLINED = "logout_outlined"
    MARK_OUTLINED = "mark_outlined"
    MAXIMIZE_WIN_OUTLINED = "maximize-win_outlined"
    MAYBE_OUTLINED = "maybe_outlined"
    MENU_OUTLINED = "menu_outlined"
    MINIMIZE_OUTLINED = "minimize_outlined"
    MINIMIZE_WIN_OUTLINED = "minimize-win_outlined"
    MORE_OUTLINED = "more_outlined"
    MORE_ADD_OUTLINED = "more-add_outlined"
    MORE_BOLD_OUTLINED = "more-bold_outlined"
    MORE_CLOSE_OUTLINED = "more-close_outlined"
    MORE_REACTION_OUTLINED = "more-reaction_outlined"
    MORE_VERTICAL_OUTLINED = "more-vertical_outlined"
    MOSAIC_OUTLINED = "mosaic_outlined"
    MOSAIC_BOX_OUTLINED = "mosaic-box_outlined"
    MOSAIC_SMEAR_OUTLINED = "mosaic-smear_outlined"
    MULTIPLE_OUTLINED = "multiple_outlined"
    MULTITASK_OUTLINED = "multitask_outlined"
    MULTITASK_1_OUTLINED = "multitask-1_outlined"
    MULTITASK_2_OUTLINED = "multitask-2_outlined"
    MULTITASK_3_OUTLINED = "multitask-3_outlined"
    MULTITASK_4_OUTLINED = "multitask-4_outlined"
    MULTITASK_5_OUTLINED = "multitask-5_outlined"
    NAVIGATE_OUTLINED = "navigate_outlined"
    NO_OUTLINED = "no_outlined"
    NOTSPAM_OUTLINED = "notspam_outlined"
    ORIGINAL_SIZE_OUTLINED = "original-size_outlined"
    ORIGINAL_SIZE_VIEW_OUTLINED = "original-size-view_outlined"
    PICTURE_IN_PICTURE_OUTLINED = "picture-in-picture_outlined"
    PULL_DOWN_OUTLINED = "pull-down_outlined"
    PULL_LEFT_OUTLINED = "pull-left_outlined"
    PULL_RIGHT_OUTLINED = "pull-right_outlined"
    QUICKSETTING_OUTLINED = "quicksetting_outlined"
    RECALL_OUTLINED = "recall_outlined"
    RECTANGLE_OUTLINED = "rectangle_outlined"
    REDUCE_OUTLINED = "reduce_outlined"
    REFRESH_OUTLINED = "refresh_outlined"
    REPEAT_OUTLINED = "repeat_outlined"
    REPLACE_OUTLINED = "replace_outlined"
    REPORT_OUTLINED = "report_outlined"
    RESIZE_OUTLINED = "resize_outlined"
    RICHTEXT_OUTLINED = "richtext_outlined"
    RICHTEXT_QUIT_OUTLINED = "richtext-quit_outlined"
    RIGHT_OUTLINED = "right_outlined"
    RIGHT_BOLD_OUTLINED = "right-bold_outlined"
    RIGHT_SMALL_CCM_OUTLINED = "right-small-ccm_outlined"
    ROTATE_OUTLINED = "rotate_outlined"
    ROTATE_SLICE_OUTLINED = "rotate-slice_outlined"
    SAFE_OUTLINED = "safe_outlined"
    SAFE_PASS_OUTLINED = "safe-pass_outlined"
    SAFE_SETTINGS_OUTLINED = "safe-settings_outlined"
    SAFE_VC_OUTLINED = "safe-vc_outlined"
    SCORE_OUTLINED = "score_outlined"
    SEARCH_OUTLINED = "search_outlined"
    SELECT_OUTLINED = "select_outlined"
    SEND_OUTLINED = "send_outlined"
    SEPARATE_OUTLINED = "separate_outlined"
    SET_LEFT_OUTLINED = "set-left_outlined"
    SETTING_OUTLINED = "setting_outlined"
    SETTING_INTER_OUTLINED = "setting-inter_outlined"
    SET_TOP_OUTLINED = "set-top_outlined"
    SET_TOP_CANCEL_OUTLINED = "set-top-cancel_outlined"
    SHARE_OUTLINED = "share_outlined"
    SHARE_LABEL_OUTLINED = "share-label_outlined"
    SIGNATURE_OUTLINED = "signature_outlined"
    SPAM_OUTLINED = "spam_outlined"
    SWITCH_OUTLINED = "switch_outlined"
    TAB_DOWN_OUTLINED = "tab-down_outlined"
    TAB_MORE_OUTLINED = "tab-more_outlined"
    THUMBDOWN_OUTLINED = "thumbdown_outlined"
    THUMBSUP_OUTLINED = "thumbsup_outlined"
    TIME_OUTLINED = "time_outlined"
    TRANSLATE_OUTLINED = "translate_outlined"
    UNMAXIMIZE_WIN_OUTLINED = "unmaximize-win_outlined"
    UNMULTITASK_OUTLINED = "unmultitask_outlined"
    UP_OUTLINED = "up_outlined"
    UP_BOLD_OUTLINED = "up-bold_outlined"
    UP_EXPAND_OUTLINED = "up-expand_outlined"
    UP_LEFT_OUTLINED = "up-left_outlined"
    UPLOAD_OUTLINED = "upload_outlined"
    UPLOAD_NEW_FILE_OUTLINED = "upload-new-file_outlined"
    UP_ROUND_OUTLINED = "up-round_outlined"
    UP_TOP_OUTLINED = "up-top_outlined"
    VIDEO_UPLOAD_OUTLINED = "video-upload_outlined"
    VIEW_3DAY_OUTLINED = "view-3day_outlined"
    VIEW_DAY_OUTLINED = "view-day_outlined"
    VIEWINCHAT_OUTLINED = "viewinchat_outlined"
    VIEW_MONTH_OUTLINED = "view-month_outlined"
    VIEW_TASK_OUTLINED = "view-task_outlined"
    VISIBLE_OUTLINED = "visible_outlined"
    VISIBLE_LOCK_OUTLINED = "visible-lock_outlined"
    VOICE_OUTLINED = "voice_outlined"
    VOICE2TEXT_OUTLINED = "voice2text_outlined"
    VOICE_TEXT_OUTLINED = "voice-text_outlined"
    W1_H1_OUTLINED = "w1-h1_outlined"
    W3_H4_OUTLINED = "w3-h4_outlined"
    W4_H3_OUTLINED = "w4-h3_outlined"
    W9_H16_OUTLINED = "w9-h16_outlined"
    W16_H9_OUTLINED = "w16-h9_outlined"
    WARNING_OUTLINED = "warning_outlined"
    WINDOW_CLOSE_OUTLINED = "window-close_outlined"
    WINDOW_MAX_OUTLINED = "window-max_outlined"
    WINDOW_MINI_OUTLINED = "window-mini_outlined"
    WINDOW_NEW_OUTLINED = "window-new_outlined"
    YES_OUTLINED = "yes_outlined"
    ZOOM_IN_OUTLINED = "zoom-in_outlined"
    ZOOM_OUT_OUTLINED = "zoom-out_outlined"
    BUILDING_OUTLINED = "building_outlined"
    BUILDING_UNAVAILABLE_OUTLINED = "building-unavailable_outlined"
    ROOM_OUTLINED = "room_outlined"
    ROOM_UNAVAILABLE_OUTLINED = "room-unavailable_outlined"
    CALENDAR_LINE_OUTLINED = "calendar-line_outlined"
    CALENDAR_SETTING_OUTLINED = "calendar-setting_outlined"
    CALENDAR_SWITCH_OUTLINED = "calendar-switch_outlined"
    CALENDAR_TITTLE_OUTLINED = "calendar-tittle_outlined"
    CALENDAR_TOMORROW_OUTLINED = "calendar-tomorrow_outlined"
    CALENDAR_VIEW_OUTLINED = "calendar-view_outlined"
    CHATBOX_OUTLINED = "chatbox_outlined"
    CHATBOX_REMOVE_OUTLINED = "chatbox-remove_outlined"
    CLOUD_OUTLINED = "cloud_outlined"
    CLOUD_CHANGE_OUTLINED = "cloud-change_outlined"
    CLOUD_DOWNLOAD_OUTLINED = "cloud-download_outlined"
    CLOUD_UPLOAD_OUTLINED = "cloud-upload_outlined"
    DISBANDGROUP_OUTLINED = "disbandgroup_outlined"
    EXTERNAL_OUTLINED = "external_outlined"
    FLAG_OUTLINED = "flag_outlined"
    FLAG_UNAVAILABLE_OUTLINED = "flag-unavailable_outlined"
    FORWARD_OUTLINED = "forward_outlined"
    FORWARD_ALL_OUTLINED = "forward-all_outlined"
    FORWARD_COM_OUTLINED = "forward-com_outlined"
    GIFT_OUTLINED = "gift_outlined"
    GIFT_BAG_OUTLINED = "gift-bag_outlined"
    GROUP_CARD_OUTLINED = "group-card_outlined"
    INBOX_OUTLINED = "inbox_outlined"
    LANGUAGE_OUTLINED = "language_outlined"
    LATER_OUTLINED = "later_outlined"
    MAIL_OUTLINED = "mail_outlined"
    MAIL_CODE_OUTLINED = "mail-code_outlined"
    MAIL_CONTACT_OUTLINED = "mail-contact_outlined"
    MAIL_SETTING_OUTLINED = "mail-setting_outlined"
    MARK_READ_OUTLINED = "mark-read_outlined"
    MASSAGE_BOX_MUTE_OUTLINED = "massage-box-mute_outlined"
    MASSAGE_BOX_OUT_OUTLINED = "massage-box-out_outlined"
    MEETING_PLAN_OUTLINED = "meeting-plan_outlined"
    MEETING_ROOM_OFFLINE_OUTLINED = "meeting-room-offline_outlined"
    MOVE_OUTLINED = "move_outlined"
    ORGANIZATION_BOOK_OUTLINED = "organization-book_outlined"
    OUTBOX_OUTLINED = "outbox_outlined"
    PRIORITY_OUTLINED = "priority_outlined"
    PRIVACY_LOCATION_OUTLINED = "privacy-location_outlined"
    REPLY_OUTLINED = "reply_outlined"
    REPLY_ALL_OUTLINED = "reply-all_outlined"
    RESOLVE_OUTLINED = "resolve_outlined"
    SENT_OUTLINED = "sent_outlined"
    SENT_CANCEL_OUTLINED = "sent-cancel_outlined"
    SENT_SCHEDULED_OUTLINED = "sent-scheduled_outlined"
    SHARELINK_OUTLINED = "sharelink_outlined"
    SUBSCRIBE_OUTLINED = "subscribe_outlined"
    SUBSCRIBE_ADD_OUTLINED = "subscribe-add_outlined"
    SUBSCRIBE_CANCEL_OUTLINED = "subscribe-cancel_outlined"
    TAG_OUTLINED = "tag_outlined"
    UNREAD_OUTLINED = "unread_outlined"
    UNTAG_OUTLINED = "untag_outlined"
    UPDATEMEETING_OUTLINED = "updatemeeting_outlined"
    WALLET_OUTLINED = "wallet_outlined"
    ADD_ENTRY_OUTLINED = "add-entry_outlined"
    ALLMAIL_OUTLINED = "allmail_outlined"
    APPROVAL_OUTLINED = "approval_outlined"
    APPSTORE_OUTLINED = "appstore_outlined"
    ARCHIVE_OUTLINED = "archive_outlined"
    ATTACHMENT_OUTLINED = "attachment_outlined"
    BONUS_PAYROLL_OUTLINED = "bonus-payroll_outlined"
    BROWSER_MAC_OUTLINED = "browser-mac_outlined"
    BROWSER_WIN_OUTLINED = "browser-win_outlined"
    BUSY_OUTLINED = "busy_outlined"
    CAL_CREATOR_OUTLINED = "cal-creator_outlined"
    CALENDAR_OUTLINED = "calendar_outlined"
    CALENDAR_1_OUTLINED = "calendar-1_outlined"
    CALENDAR_2_OUTLINED = "calendar-2_outlined"
    CALENDAR_3_OUTLINED = "calendar-3_outlined"
    CALENDAR_4_OUTLINED = "calendar-4_outlined"
    CALENDAR_5_OUTLINED = "calendar-5_outlined"
    CALENDAR_6_OUTLINED = "calendar-6_outlined"
    CALENDAR_7_OUTLINED = "calendar-7_outlined"
    CALENDAR_8_OUTLINED = "calendar-8_outlined"
    CALENDAR_9_OUTLINED = "calendar-9_outlined"
    CALENDAR_10_OUTLINED = "calendar-10_outlined"
    CALENDAR_11_OUTLINED = "calendar-11_outlined"
    CALENDAR_12_OUTLINED = "calendar-12_outlined"
    CALENDAR_13_OUTLINED = "calendar-13_outlined"
    CALENDAR_14_OUTLINED = "calendar-14_outlined"
    CALENDAR_15_OUTLINED = "calendar-15_outlined"
    CALENDAR_16_OUTLINED = "calendar-16_outlined"
    CALENDAR_17_OUTLINED = "calendar-17_outlined"
    CALENDAR_18_OUTLINED = "calendar-18_outlined"
    CALENDAR_19_OUTLINED = "calendar-19_outlined"
    CALENDAR_20_OUTLINED = "calendar-20_outlined"
    CALENDAR_21_OUTLINED = "calendar-21_outlined"
    CALENDAR_22_OUTLINED = "calendar-22_outlined"
    CALENDAR_23_OUTLINED = "calendar-23_outlined"
    CALENDAR_24_OUTLINED = "calendar-24_outlined"
    CALENDAR_25_OUTLINED = "calendar-25_outlined"
    CALENDAR_26_OUTLINED = "calendar-26_outlined"
    CALENDAR_27_OUTLINED = "calendar-27_outlined"
    CALENDAR_28_OUTLINED = "calendar-28_outlined"
    CALENDAR_29_OUTLINED = "calendar-29_outlined"
    CALENDAR_30_OUTLINED = "calendar-30_outlined"
    CALENDAR_31_OUTLINED = "calendar-31_outlined"
    CALENDAR_ADD_OUTLINED = "calendar-add_outlined"
    CALENDAR_CHAT_OUTLINED = "calendar-chat_outlined"
    CALENDAR_DATE_OUTLINED = "calendar-date_outlined"
    CALENDAR_DONE_OUTLINED = "calendar-done_outlined"
    CALENDAR_EDIT_OUTLINED = "calendar-edit_outlined"
    ADD_CHAT_OUTLINED = "add-chat_outlined"
    ADD_COMMENT_OUTLINED = "add-comment_outlined"
    AT_OUTLINED = "at_outlined"
    BOARDS_OUTLINED = "boards_outlined"
    BUZZ_OUTLINED = "buzz_outlined"
    CANCEL_BUZZ_OUTLINED = "cancel-buzz_outlined"
    CHAT_OUTLINED = "chat_outlined"
    CHAT_ALL_OUTLINED = "chat-all_outlined"
    CHAT_DONE_OUTLINED = "chat-done_outlined"
    CHAT_FORBIDDEN_OUTLINED = "chat-forbidden_outlined"
    CHAT_LEFT_OUTLINED = "chat-left_outlined"
    CHAT_NEWS_OUTLINED = "chat-news_outlined"
    CHAT_RIGHT_OUTLINED = "chat-right_outlined"
    CHAT_SECRET_OUTLINED = "chat-secret_outlined"
    CHAT_TOPIC_OUTLINED = "chat-topic_outlined"
    COMMENT_SUBSCRIBE_OUTLINED = "comment-subscribe_outlined"
    DISCUSS_OUTLINED = "discuss_outlined"
    FOLLOWING_OUTLINED = "following_outlined"
    HASHTAG_OUTLINED = "hashtag_outlined"
    MESSAGE_MUTE_OUTLINED = "message-mute_outlined"
    MOMENTS_CATEGORIES_OUTLINED = "moments-categories_outlined"
    REPLY_CN_OUTLINED = "reply-cn_outlined"
    SHOW_FEED_OUTLINED = "show-feed_outlined"
    STOP_DEMOTE_OUTLINED = "stop-demote_outlined"
    STOP_PROMOTE_OUTLINED = "stop-promote_outlined"
    VOTE_OUTLINED = "vote_outlined"
    ADMIN_CANCEL_OUTLINED = "admin-cancel_outlined"
    ADMIN_SETTING_OUTLINED = "admin-setting_outlined"
    ASSIGNED_OUTLINED = "assigned_outlined"
    COMMUNITY_TAB_OUTLINED = "community-tab_outlined"
    CONTACTS_OUTLINED = "contacts_outlined"
    EMOJI_OUTLINED = "emoji_outlined"
    EMOJI_ADD_OUTLINED = "emoji-add_outlined"
    GROUP_OUTLINED = "group_outlined"
    MAIL_COLLABORATOR_OUTLINED = "mail-collaborator_outlined"
    MEMBER_OUTLINED = "member_outlined"
    MEMBER_ADD_OUTLINED = "member-add_outlined"
    MEMBER_NEW_OUTLINED = "member-new_outlined"
    NEARBY_GROUP_OUTLINED = "nearby-group_outlined"
    PERSON_ADMIT_OUTLINED = "person-admit_outlined"
    PERSON_REMOVE_OUTLINED = "person-remove_outlined"
    ROBOT_OUTLINED = "robot_outlined"
    ROBOT_ADD_OUTLINED = "robot-add_outlined"
    SPECIAL_ATTENTION_OUTLINED = "special-attention_outlined"
    TEAM_ADD_OUTLINED = "team-add_outlined"
    TEAM_CODE_OUTLINED = "team-code_outlined"
    WITHDRAWCOHOST_OUTLINED = "withdrawcohost_outlined"
    ACTIVE_SPEAKER_VIEW_OUTLINED = "active-speaker-view_outlined"
    ADMIN_MULTISTAGE_OUTLINED = "admin-multistage_outlined"
    ANNOTATE_OUTLINED = "annotate_outlined"
    ANNOUNCE_OUTLINED = "announce_outlined"
    ASTERISK_OUTLINED = "asterisk_outlined"
    AUTO_LAYOUT_OUTLINED = "auto-layout_outlined"
    AV_SET_DOWN_OUTLINED = "av-set-down_outlined"
    AV_SET_UP_OUTLINED = "av-set-up_outlined"
    BARRAGE_OFF_OUTLINED = "barrage-off_outlined"
    BIG_EYE_OUTLINED = "big-eye_outlined"
    BLUETOOTH_OUTLINED = "bluetooth_outlined"
    BLUR_OUTLINED = "blur_outlined"
    BREAKOUTROOMS_OUTLINED = "breakoutrooms_outlined"
    CALL_OUTLINED = "call_outlined"
    CALL_END_OUTLINED = "call-end_outlined"
    CALL_NET_OUTLINED = "call-net_outlined"
    CALL_SECRET_OUTLINED = "call-secret_outlined"
    CALL_VIDEO_OUTLINED = "call-video_outlined"
    CAMERA_FLIP_OUTLINED = "camera-flip_outlined"
    CONTROLLER_OUTLINED = "controller_outlined"
    CURSOR_OUTLINED = "cursor_outlined"
    DIALPAD_OUTLINED = "dialpad_outlined"
    EAR_OUTLINED = "ear_outlined"
    EFFECTS_OUTLINED = "effects_outlined"
    EMOJI_KEYBOARD_OUTLINED = "emoji-keyboard_outlined"
    FACE_LIFT_OUTLINED = "face-lift_outlined"
    FLOATING_VIEW_OUTLINED = "floating-view_outlined"
    GRID_VIEW_OUTLINED = "grid-view_outlined"
    HASH_OUTLINED = "hash_outlined"
    HIDE_TOOLBAR_OUTLINED = "hide-toolbar_outlined"
    HIGHLIGHTER_OUTLINED = "highlighter_outlined"
    HOST_CONTROL_OUTLINED = "host-control_outlined"
    LABS_OUTLINED = "labs_outlined"
    LASER_OUTLINED = "laser_outlined"
    LEAVEROOM_OUTLINED = "leaveroom_outlined"
    LIPSTICK_OUTLINED = "lipstick_outlined"
    LIVESTREAM_OUTLINED = "livestream_outlined"
    LIVESTREAM_CONTENT_OUTLINED = "livestream-content_outlined"
    LIVESTREAM_HYBRID_OUTLINED = "livestream-hybrid_outlined"
    LIVESTREAM_START_OUTLINED = "livestream-start_outlined"
    LOCK_OUTLINED = "lock_outlined"
    LOTTERY_BAG_OUTLINED = "lottery-bag_outlined"
    LOWERHAND_OUTLINED = "lowerhand_outlined"
    MIC_OUTLINED = "mic_outlined"
    MIC_OFF_OUTLINED = "mic-off_outlined"
    MULTISTREAMING_OUTLINED = "multistreaming_outlined"
    NEW_JOIN_MEETING_OUTLINED = "new-join-meeting_outlined"
    OFFICEPHONE_OUTLINED = "officephone_outlined"
    PAUSE_LIVESTREAM_OUTLINED = "pause-livestream_outlined"
    PEN_OUTLINED = "pen_outlined"
    PODCAST_OUTLINED = "podcast_outlined"
    RAISEHAND_OUTLINED = "raisehand_outlined"
    RECORD_OUTLINED = "record_outlined"
    SCENE_EXPAND_OUTLINED = "scene-expand_outlined"
    SCENE_MINIMAZE_OUTLINED = "scene-minimaze_outlined"
    SCENE_REUPLOAD_OUTLINED = "scene-reupload_outlined"
    SEPWINDOW_OUTLINED = "sepwindow_outlined"
    SHARE_COMPUTER_AUDIO_OUTLINED = "share-computer-audio_outlined"
    SHARE_SCREEN_OUTLINED = "share-screen_outlined"
    SHOW_TOOLBAR_OUTLINED = "show-toolbar_outlined"
    SIP_OUTLINED = "sip_outlined"
    SPEAKER_OUTLINED = "speaker_outlined"
    SPEAKER_MUTE_OUTLINED = "speaker-mute_outlined"
    SPEAKING_OUTLINED = "speaking_outlined"
    STOP_OUTLINED = "stop_outlined"
    STOP_SHARE_SCREEN_OUTLINED = "stop-share-screen_outlined"
    SUBMIT_FEEDBACK_OUTLINED = "submit-feedback_outlined"
    SUBTITLES_OUTLINED = "subtitles_outlined"
    SUBTITLES_SETTINGS_OUTLINED = "subtitles-settings_outlined"
    TOUCH_UP_OUTLINED = "touch-up_outlined"
    VC_SEPWINDOW_OUTLINED = "vc-sepwindow_outlined"
    VERSION_OUTLINED = "version_outlined"
    VIDEO_OUTLINED = "video_outlined"
    VIDEO_OFF_OUTLINED = "video-off_outlined"
    VIRTUAL_AVATAR_OUTLINED = "virtual-avatar_outlined"
    VIRTUAL_BG_OUTLINED = "virtual-bg_outlined"
    VIRTUAL_OFFICE_OUTLINED = "virtual-office_outlined"
    WATCH_PAGE_SETTINGS_OUTLINED = "watch-page-settings_outlined"
    APP_OUTLINED = "app_outlined"
    CHAT_PIN_OUTLINED = "chat-pin_outlined"
    INTERNET_OUTLINED = "internet_outlined"
    LOCAL_OUTLINED = "local_outlined"
    PIN_OUTLINED = "pin_outlined"
    TIME_ZONE_OUTLINED = "time-zone_outlined"
    UNPIN_OUTLINED = "unpin_outlined"
    CANCEL_OFFLINE_OUTLINED = "cancel-offline_outlined"
    DAY_OUTLINED = "day_outlined"
    NIGHT_OUTLINED = "night_outlined"
    OFFLINE_OUTLINED = "offline_outlined"
    JIRA_ISSUE_OUTLINED = "jira-issue_outlined"
    PLATFORM_OUTLINED = "platform_outlined"
    ALARM_OUTLINED = "alarm_outlined"
    BURNLIFE_DAY_OUTLINED = "burnlife-day_outlined"
    BURNLIFE_HOUR_OUTLINED = "burnlife-hour_outlined"
    BURNLIFE_MINUTE_OUTLINED = "burnlife-minute_outlined"
    BURNLIFE_NOTIME_OUTLINED = "burnlife-notime_outlined"
    BURNLIFE_WEEK_OUTLINED = "burnlife-week_outlined"
    CELLPHONE_OUTLINED = "cellphone_outlined"
    CLEAR_OUTLINED = "clear_outlined"
    COMPUTER_OUTLINED = "computer_outlined"
    FILELINK_SAVE_OUTLINED = "filelink-save_outlined"
    KEYBOARD_OUTLINED = "keyboard_outlined"
    KEYBOARD_DISPLAY_OUTLINED = "keyboard-display_outlined"
    LAN_OUTLINED = "lan_outlined"
    MOUSE_OUTLINED = "mouse_outlined"
    PC_OUTLINED = "pc_outlined"
    PHONE_OUTLINED = "phone_outlined"
    PRINTER_OUTLINED = "printer_outlined"
    QR_OUTLINED = "qr_outlined"
    RAM_OUTLINED = "ram_outlined"
    SCAN_OUTLINED = "scan_outlined"
    SIGNAL_OUTLINED = "signal_outlined"
    ADD_DOC_OUTLINED = "add-doc_outlined"
    ADDNEW_OUTLINED = "addnew_outlined"
    ADJUSTMENT_OUTLINED = "adjustment_outlined"
    ADVANCED_ACTIVED_OUTLINED = "advanced-actived_outlined"
    ALIGN_BUTTON_OUTLINED = "align-button_outlined"
    ALIGN_CENTER_OUTLINED = "align-center_outlined"
    ALIGN_LEFT_OUTLINED = "align-left_outlined"
    ALIGN_MIDDLE_OUTLINED = "align-middle_outlined"
    ALIGN_RIGHT_OUTLINED = "align-right_outlined"
    ALIGN_TOP_OUTLINED = "align-top_outlined"
    ARRANGE_OUTLINED = "arrange_outlined"
    BITABLE_AUTHORIZATION_OUTLINED = "bitable-authorization_outlined"
    BITABLEFORM_OUTLINED = "bitableform_outlined"
    BITABLEGALLERY_OUTLINED = "bitablegallery_outlined"
    BITABLEGANTT_OUTLINED = "bitablegantt_outlined"
    BITABLEGRID_OUTLINED = "bitablegrid_outlined"
    BITABLEKANBAN_OUTLINED = "bitablekanban_outlined"
    BITABLE_MOVE_DOWN_OUTLINED = "bitable-move-down_outlined"
    BITABLE_MOVE_UP_OUTLINED = "bitable-move-up_outlined"
    BITABLEUNKNOW_OUTLINED = "bitableunknow_outlined"
    BLOCKNEW_OUTLINED = "blocknew_outlined"
    BOLD_OUTLINED = "bold_outlined"
    BORDER_OUTLINED = "border_outlined"
    BORDERS_OUTLINED = "borders_outlined"
    BOTTOM_ALIGN_OUTLINED = "bottom-align_outlined"
    BRUSH_BOLD_OUTLINED = "brush-bold_outlined"
    BRUSH_LIGHT_OUTLINED = "brush-light_outlined"
    BRUSH_MEDIUM_OUTLINED = "brush-medium_outlined"
    BULK_ADD_OUTLINED = "bulk-add_outlined"
    CALLOUT_OUTLINED = "callout_outlined"
    CANCEL_FREEZE_OUTLINED = "cancel-freeze_outlined"
    CANCEL_LINK_OUTLINED = "cancel-link_outlined"
    CANCEL_TOP_ALIGN_OUTLINED = "cancel-top-align_outlined"
    CARD_OUTLINED = "card_outlined"
    CARD_SEARCH_OUTLINED = "card-search_outlined"
    CARD_VIEW_OUTLINED = "card-view_outlined"
    CCM_EDIT_OUTLINED = "ccm-edit_outlined"
    CCM_EDIT_CONTINUE_OUTLINED = "ccm-edit-continue_outlined"
    CCM_NEXT_OUTLINED = "ccm-next_outlined"
    CCM_PREVIOUS_OUTLINED = "ccm-previous_outlined"
    CCM_RENAME_OUTLINED = "ccm-rename_outlined"
    CCM_UPLOAD_OUTLINED = "ccm-upload_outlined"
    CENTER_ALIGNMENT_OUTLINED = "center-alignment_outlined"
    CLIP_OUTLINED = "clip_outlined"
    CODE_OUTLINED = "code_outlined"
    CODEBLOCK_OUTLINED = "codeblock_outlined"
    COLUMNWIDTH_OUTLINED = "columnwidth_outlined"
    COMMAND_OUTLINED = "command_outlined"
    COPY_OUTLINED = "copy_outlined"
    COVER_SETTING_OUTLINED = "cover-setting_outlined"
    CREAT_FOLDER_OUTLINED = "creat-folder_outlined"
    DATA_SHEET_OUTLINED = "data-sheet_outlined"
    DESCRIBE_OUTLINED = "describe_outlined"
    DISORDER_LIST_OUTLINED = "disorder-list_outlined"
    DIVIDER_OUTLINED = "divider_outlined"
    DOC_CHECKLIST_OUTLINED = "doc-checklist_outlined"
    DOC_COMMENT_OUTLINED = "doc-comment_outlined"
    DOC_REPLACE_OUTLINED = "doc-replace_outlined"
    DOC_SEARCH_OUTLINED = "doc-search_outlined"
    DOWN_BORDERS_OUTLINED = "down-borders_outlined"
    DRAG_OUTLINED = "drag_outlined"
    DRAWIO_OUTLINED = "drawio_outlined"
    DRIVELOAD_OUTLINED = "driveload_outlined"
    EDIT_DISCRIPTION_OUTLINED = "edit-discription_outlined"
    EDUCATION_OUTLINED = "education_outlined"
    EFFICIENCY_OUTLINED = "efficiency_outlined"
    ERASER_OUTLINED = "eraser_outlined"
    EXPAND_ITEMS_OUTLINED = "expand-items_outlined"
    FEEDBACK_OUTLINED = "feedback_outlined"
    FEISHUCLIP_OUTLINED = "feishuclip_outlined"
    FIGURE_OUTLINED = "figure_outlined"
    FILE_BITABLE_OUTLINED = "file-bitable_outlined"
    FILE_LINK_AUDIO_OUTLINED = "file-link-audio_outlined"
    FILE_LINK_AUDIO_SHORTCUT_OUTLINED = "file-link-audio-shortcut_outlined"
    FILE_LINK_BITABLE_OUTLINED = "file-link-bitable_outlined"
    FILELINK_BITABLE_SHORTCUT_OUTLINED = "filelink-bitable-shortcut_outlined"
    FILE_LINK_DETA_OUTLINED = "file-link-deta_outlined"
    FILE_LINK_DOCX_OUTLINED = "file-link-docx_outlined"
    FILE_LINK_DOCX_SHORTCUT_OUTLINED = "file-link-docx-shortcut_outlined"
    FILE_LINK_EXCEL_OUTLINED = "file-link-excel_outlined"
    FILE_LINK_EXCEL_SHORTCUT_OUTLINED = "file-link-excel-shortcut_outlined"
    FILE_LINK_IMAGE_OUTLINED = "file-link-image_outlined"
    FILE_LINK_IMAGE_SHORTCUT_OUTLINED = "file-link-image-shortcut_outlined"
    FILE_LINK_MINDNOTE_OUTLINED = "file-link-mindnote_outlined"
    FILE_LINK_OTHERFILE_OUTLINED = "file-link-otherfile_outlined"
    FILE_LINK_OTHERFILE_SHORTCUT_OUTLINED = "file-link-otherfile-shortcut_outlined"
    FILE_LINK_PDF_OUTLINED = "file-link-pdf_outlined"
    FILE_LINK_PDF_SHORTCUT_OUTLINED = "file-link-pdf-shortcut_outlined"
    FILE_LINK_PPT_OUTLINED = "file-link-ppt_outlined"
    FILE_LINK_PPT_SHORTCUT_OUTLINED = "file-link-ppt-shortcut_outlined"
    FILE_LINK_SHEET_OUTLINED = "file-link-sheet_outlined"
    FILE_LINK_SLIDE_OUTLINED = "file-link-slide_outlined"
    FILE_LINK_SLIDE_SHORTCUT_OUTLINED = "file-link-slide-shortcut_outlined"
    FILE_LINK_TEXT_OUTLINED = "file-link-text_outlined"
    FILE_LINK_TEXT_SHORTCUT_OUTLINED = "file-link-text-shortcut_outlined"
    FILE_LINK_UNKNOW_OUTLINED = "file-link-unknow_outlined"
    FILE_LINK_UNKNOW_SHORTCUT_OUTLINED = "file-link-unknow-shortcut_outlined"
    FILE_LINK_VIDEO_OUTLINED = "file-link-video_outlined"
    FILE_LINK_VIDEO_SHORTCUT_OUTLINED = "file-link-video-shortcut_outlined"
    FILE_LINK_WIKI_OUTLINED = "file-link-wiki_outlined"
    FILE_LINK_WIKI_SHORTCUT_OUTLINED = "file-link-wiki-shortcut_outlined"
    FILE_LINK_WORD_OUTLINED = "file-link-word_outlined"
    FILE_LINK_WORD2_OUTLINED = "file-link-word2_outlined"
    FILE_LINK_WORD2_SHORTCUT_OUTLINED = "file-link-word2-shortcut_outlined"
    FILE_LINK_ZIP_OUTLINED = "file-link-zip_outlined"
    FILE_LINK_ZIP_SHORTCUT_OUTLINED = "file-link-zip-shortcut_outlined"
    FIND_AND_REPLACE_OUTLINED = "find-and-replace_outlined"
    FLIP_HORIZONTALLY_OUTLINED = "flip-horizontally_outlined"
    FLIP_VERTICALLY_OUTLINED = "flip-vertically_outlined"
    FOLDER_OUTLINED = "folder_outlined"
    FONTCOLOR_OUTLINED = "fontcolor_outlined"
    FORM_BRUSH_OUTLINED = "form-brush_outlined"
    FORMULA_OUTLINED = "formula_outlined"
    FRAME_SELECTION_OUTLINED = "frame-selection_outlined"
    FREEZE_1_COLUMN_OUTLINED = "freeze-1-column_outlined"
    FREEZE_COLUMN_AND_ROW_OUTLINED = "freeze-column-and-row_outlined"
    FREEZE_ROW_OUTLINED = "freeze-row_outlined"
    FULL_WIDTH_OUTLINED = "full-width_outlined"
    FUNCTIONS_OUTLINED = "functions_outlined"
    GANTTSET_OUTLINED = "ganttset_outlined"
    GLOBAL_TRIANGLESMALL_OUTLINED = "global-trianglesmall_outlined"
    GRIDLINES_OUTLINED = "gridlines_outlined"
    GRIDLINES_HIDE_OUTLINED = "gridlines-hide_outlined"
    GROUP_SELECTION_OUTLINED = "group-selection_outlined"
    H1_OUTLINED = "h1_outlined"
    H2_OUTLINED = "h2_outlined"
    H3_OUTLINED = "h3_outlined"
    H4_OUTLINED = "h4_outlined"
    H5_OUTLINED = "h5_outlined"
    H6_OUTLINED = "h6_outlined"
    H7_OUTLINED = "h7_outlined"
    H8_OUTLINED = "h8_outlined"
    H9_OUTLINED = "h9_outlined"
    HELPDESK_OUTLINED = "helpdesk_outlined"
    HELPDESK_OFF_OUTLINED = "helpdesk-off_outlined"
    HIGHLIGHTRANKS_OUTLINED = "highlightranks_outlined"
    HIGH_LIGHT_REPEATVALUE_OUTLINED = "high-light-repeatvalue_outlined"
    HISTORY_COMMANT_OUTLINED = "history-commant_outlined"
    HN_OUTLINED = "hn_outlined"
    HOME_OUTLINED = "home_outlined"
    HORIZONTAL_LINE_OUTLINED = "horizontal-line_outlined"
    IMAGE_REPLACEMENT_OUTLINED = "image-replacement_outlined"
    INCREASE_INDENTATION_OUTLINED = "increase-indentation_outlined"
    INNER_BORDERS_OUTLINED = "inner-borders_outlined"
    INSERT_CHART_OUTLINED = "insert-chart_outlined"
    INSERT_COLUMN_LEFT_OUTLINED = "insert-column-left_outlined"
    INSERT_COLUMN_RIGHT_OUTLINED = "insert-column-right_outlined"
    INSERT_DOWN_OUTLINED = "insert-down_outlined"
    INSERT_LEFT_OUTLINED = "insert-left_outlined"
    INSERT_RIGHT_OUTLINED = "insert-right_outlined"
    INSERT_ROW_DOWN_OUTLINED = "insert-row-down_outlined"
    INSERT_ROW_UP_OUTLINED = "insert-row-up_outlined"
    INSERT_UP_OUTLINED = "insert-up_outlined"
    INTO_ITEM_OUTLINED = "into-item_outlined"
    ITALIC_OUTLINED = "italic_outlined"
    JUSTIFY_TEXT_OUTLINED = "justify-text_outlined"
    LATEX_OUTLINED = "latex_outlined"
    LEFT_ALIGNMENT_OUTLINED = "left-alignment_outlined"
    LEFT_BORDERS_OUTLINED = "left-borders_outlined"
    LINK_RECORD_OUTLINED = "link-record_outlined"
    LIST_OUTLINED = "list_outlined"
    LIST_VIEW_OUTLINED = "list-view_outlined"
    LONG_FIGURE_OUTLINED = "long-figure_outlined"
    LOOKUP_OUTLINED = "lookup_outlined"
    MAGNIFY_OUTLINED = "magnify_outlined"
    MAGNIFY_LEFT_OUTLINED = "magnify-left_outlined"
    MERGECELLS_OUTLINED = "mergecells_outlined"
    MINDMAP_OUTLINED = "mindmap_outlined"
    MINDMAP_DOWN_OUTLINED = "mindmap-down_outlined"
    MINDMAP_LEFT_OUTLINED = "mindmap-left_outlined"
    MINDMAP_RIGHT_OUTLINED = "mindmap-right_outlined"
    MINDMAP_SIDE_OUTLINED = "mindmap-side_outlined"
    MINIFY_OUTLINED = "minify_outlined"
    MOVE_DOWN_OUTLINED = "move-down_outlined"
    MOVE_TO_OUTLINED = "move-to_outlined"
    MOVE_UP_OUTLINED = "move-up_outlined"
    MULTI_ELEMENTALIGN_BOTTOM_OUTLINED = "multi-elementalign-bottom_outlined"
    MULTI_ELEMENTALIGN_CENTER_OUTLINED = "multi-elementalign-center_outlined"
    MULTI_ELEMENTALIGN_LEFT_OUTLINED = "multi-elementalign-left_outlined"
    MULTI_ELEMENTALIGN_MIDDLE_OUTLINED = "multi-elementalign-middle_outlined"
    MULTI_ELEMENTALIGN_RIGHT_OUTLINED = "multi-elementalign-right_outlined"
    MULTI_ELEMENTALIGN_TOP_OUTLINED = "multi-elementalign-top_outlined"
    MYFOLDER_OUTLINED = "myfolder_outlined"
    NAVIGATION_BUTTON_OUTLINED = "navigation-button_outlined"
    NEW_SHORTCUT_OUTLINED = "new-shortcut_outlined"
    NO_BORDERS_OUTLINED = "no-borders_outlined"
    NUMBER_OUTLINED = "number_outlined"
    NUMBER_00_OUTLINED = "number-00_outlined"
    NUMBER_0_OUTLINED = "number-0_outlined"
    NUMBERED_LIST_CONTINUED_OUTLINED = "numbered-list-continued_outlined"
    NUMBERED_LIST_NEW_OUTLINED = "numbered-list-new_outlined"
    NUMBERED_LIST_RESTART_OUTLINED = "numbered-list-restart_outlined"
    OKR_OUTLINED = "okr_outlined"
    OPERATIONRECORD_OUTLINED = "operationrecord_outlined"
    ORDER_LIST_OUTLINED = "order-list_outlined"
    ORGANIZATION_OUTLINED = "organization_outlined"
    OUTER_BORDERS_OUTLINED = "outer-borders_outlined"
    OUTLINE_OUTLINED = "outline_outlined"
    PAPERBAG_OUTLINED = "paperbag_outlined"
    PASTE_OUTLINED = "paste_outlined"
    PLUGIN_OUTLINED = "plugin_outlined"
    POLL_OUTLINED = "poll_outlined"
    PRESENTATION_OUTLINED = "presentation_outlined"
    PREVIEW_OUTLINED = "preview_outlined"
    PREVIEW_VIEW_OUTLINED = "preview-view_outlined"
    PROTECT_RANGE_OUTLINED = "protect-range_outlined"
    READINFO_OUTLINED = "readinfo_outlined"
    REDO_OUTLINED = "redo_outlined"
    REDUCE_INDENTATION_OUTLINED = "reduce-indentation_outlined"
    REFERENCE_OUTLINED = "reference_outlined"
    RETURN_ITEM_OUTLINED = "return-item_outlined"
    REVOLVE_OUTLINED = "revolve_outlined"
    RIGHT_ALIGNMENT_OUTLINED = "right-alignment_outlined"
    RIGHT_BORDERS_OUTLINED = "right-borders_outlined"
    ROW_EXTRATALL_OUTLINED = "row-extratall_outlined"
    ROW_HEIGHT_OUTLINED = "row-height_outlined"
    ROWHEIGHTSHEET_OUTLINED = "rowheightsheet_outlined"
    ROW_MEDIUM_OUTLINED = "row-medium_outlined"
    ROW_TALL_OUTLINED = "row-tall_outlined"
    SCREEN_ROTATION_OUTLINED = "screen-rotation_outlined"
    SCREENSHOTS_OUTLINED = "screenshots_outlined"
    SEARCH_OUTLINE_OUTLINED = "search-outline_outlined"
    SELECT_ALL_OUTLINED = "select-all_outlined"
    SELECT_CHARACTER_OUTLINED = "select-character_outlined"
    SELECT_DOWN_OUTLINED = "select-down_outlined"
    SELECT_LINE_OUTLINED = "select-line_outlined"
    SELECT_UP_OUTLINED = "select-up_outlined"
    SHEET_BAR_OUTLINED = "sheet-bar_outlined"
    SHEET_BITABLE_OUTLINED = "sheet-bitable_outlined"
    SHEET_CARDMODEL_OUTLINED = "sheet-cardmodel_outlined"
    SHEET_COMBO_OUTLINED = "sheet-combo_outlined"
    SHEET_CONDITIONALFORMAT_OUTLINED = "sheet-conditionalformat_outlined"
    SHEET_CURRENCY_OUTLINED = "sheet-currency_outlined"
    SHEET_DATAREFERENCE_OUTLINED = "sheet-datareference_outlined"
    SHEET_DOLLAR_OUTLINED = "sheet-dollar_outlined"
    SHEET_FONTMINUS_OUTLINED = "sheet-fontminus_outlined"
    SHEET_FONTPLUS_OUTLINED = "sheet-fontplus_outlined"
    SHEET_GAUGE_OUTLINED = "sheet-gauge_outlined"
    SHEET_HISTOGRAM_OUTLINED = "sheet-histogram_outlined"
    SHEET_LINE_OUTLINED = "sheet-line_outlined"
    SHEET_MENU_OUTLINED = "sheet-menu_outlined"
    SHEET_ONEDATAREFERENCE_OUTLINED = "sheet-onedatareference_outlined"
    SHEET_PARETO_OUTLINED = "sheet-pareto_outlined"
    SHEET_PERCENT_OUTLINED = "sheet-percent_outlined"
    SHEET_PIE_OUTLINED = "sheet-pie_outlined"
    SHEET_SCATTER_OUTLINED = "sheet-scatter_outlined"
    SHEET_SIGN_OUTLINED = "sheet-sign_outlined"
    SHEET_SLASH_OUTLINED = "sheet-slash_outlined"
    SHEET_TABLE_OUTLINED = "sheet-table_outlined"
    SHEET_TEXT_COLUMNS_OUTLINED = "sheet-text-columns_outlined"
    SHEET_WORDCLOUD_OUTLINED = "sheet-wordcloud_outlined"
    SIDEBAR_OUTLINED = "sidebar_outlined"
    SIDE_EXPAND_OUTLINED = "side-expand_outlined"
    SIDE_FOLD_OUTLINED = "side-fold_outlined"
    SLIDES_MODE_OUTLINED = "slides-mode_outlined"
    SORT_A_TO_Z_OUTLINED = "sort-a-to-z_outlined"
    SORT_Z_TO_A_OUTLINED = "sort-z-to-a_outlined"
    SPACE_OUTLINED = "space_outlined"
    SPACE_DOWN_OUTLINED = "space-down_outlined"
    SPACE_DOWN_BOLD_OUTLINED = "space-down-bold_outlined"
    SPACE_GROUP_OUTLINED = "space-group_outlined"
    SPACE_SHAREDFOLDER_OUTLINED = "space-sharedfolder_outlined"
    SPACE_UP_OUTLINED = "space-up_outlined"
    SPACE_UP_BOLD_OUTLINED = "space-up-bold_outlined"
    STORAGE_OUTLINED = "storage_outlined"
    STYLE_OUTLINED = "style_outlined"
    STYLE_FILLCOLOR_OUTLINED = "style-fillcolor_outlined"
    STYLE_FONTCOLOR_OUTLINED = "style-fontcolor_outlined"
    STYLE_HIGHTLIGHT_OUTLINED = "style-hightlight_outlined"
    STYLE_SET_OUTLINED = "style-set_outlined"
    SWITCH_COMPLETION_OUTLINED = "switch-completion_outlined"
    SWITCH_ITEM_OUTLINED = "switch-item_outlined"
    TABLE_GROUP_OUTLINED = "table-group_outlined"
    TAKE_ACTION_OUTLINED = "take-action_outlined"
    TEMPLATE_OUTLINED = "template_outlined"
    TEXT_OUTLINED = "text_outlined"
    TEXT2_OUTLINED = "text2_outlined"
    TEXT_AA_OUTLINED = "text-aa_outlined"
    TEXTSTYLE_OUTLINED = "textstyle_outlined"
    TODO_OUTLINED = "todo_outlined"
    TOOL_ADDBLOCK_OUTLINED = "tool-addblock_outlined"
    TOOL_INDENT_OUTLINED = "tool-indent_outlined"
    TOOL_OUTDENT_OUTLINED = "tool-outdent_outlined"
    TOP_ALIGN_OUTLINED = "top-align_outlined"
    TRANSFERSHEET_OUTLINED = "transfersheet_outlined"
    TREELIST_OUTLINED = "treelist_outlined"
    TUODONG_OUTLINED = "tuodong_outlined"
    TYPOGRAPHY_OUTLINED = "typography_outlined"
    UML_OUTLINED = "uml_outlined"
    UNDERLINE_OUTLINED = "underline_outlined"
    UNDO_OUTLINED = "undo_outlined"
    UNTEXT_OUTLINED = "untext_outlined"
    UP_BORDERS_OUTLINED = "up-borders_outlined"
    UPDATE_LOG_OUTLINED = "update-log_outlined"
    UPLOADFILE_OUTLINED = "uploadfile_outlined"
    UPLOADFOLDER_OUTLINED = "uploadfolder_outlined"
    UX_FILL_COLOR_OUTLINED = "ux-fill-color_outlined"
    UX_FONT_COLOR_OUTLINED = "ux-font-color_outlined"
    UX_HIGHLIGHT_OUTLINED = "ux-highlight_outlined"
    VC_DRAG_OUTLINED = "vc-drag_outlined"
    VERTICAL_ALIGN_OUTLINED = "vertical-align_outlined"
    VIEW_OUTLINED = "view_outlined"
    VIEW_LIST_OUTLINED = "view-list_outlined"
    WARN_REPORT_OUTLINED = "warn-report_outlined"
    WEB_CARD_OUTLINED = "web-card_outlined"
    WHITEBOARD_OUTLINED = "whiteboard_outlined"
    WIKI_OUTLINED = "wiki_outlined"
    WIKI_BOOK_OUTLINED = "wiki-book_outlined"
    WIKI_DOC_SHORTCUT_OUTLINED = "wiki-doc-shortcut_outlined"
    WIKILINK_OUTLINED = "wikilink_outlined"
    WIKI_MINDNOTE_SHORTCUT_OUTLINED = "wiki-mindnote-shortcut_outlined"
    WIKI_SHEET_SHORTCUT_OUTLINED = "wiki-sheet-shortcut_outlined"
    WIKITREELISTBLOD_OUTLINED = "wikitreelistblod_outlined"
    WORKSPACE_OUTLINED = "workspace_outlined"
    WRAP_OUTLINED = "wrap_outlined"
    WRAPPING_OUTLINED = "wrapping_outlined"
    ADD_DICTIONARY_OUTLINED = "add-dictionary_outlined"
    GOOGLEDRIVE_OUTLINED = "googledrive_outlined"
    HIRELOGO_OUTLINED = "hirelogo_outlined"
    JIRA_OUTLINED = "jira_outlined"
    LARK_OUTLINED = "lark_outlined"
    LARKFRIENDS_OUTLINED = "larkfriends_outlined"
    MEEGO_OUTLINED = "meego_outlined"
    ADD_BUTTON_OUTLINED = "add-button_outlined"
    ADD_SHEET_OUTLINED = "add-sheet_outlined"
    ADD_TRUSTPARTY_OUTLINED = "add-trustparty_outlined"
    BITABLE_LOCK_OUTLINED = "bitable-lock_outlined"
    BRIEFCASE_OUTLINED = "briefcase_outlined"
    BRING_FRONT_OUTLINED = "bring-front_outlined"
    BUFFING_OUTLINED = "buffing_outlined"
    BUSINESS_CARD_OUTLINED = "business-card_outlined"
    CANCEL_RICHTEXT_OUTLINED = "cancel-richtext_outlined"
    CCM_FILTER_OUTLINED = "ccm-filter_outlined"
    CCM_OUTBOX_OUTLINED = "ccm-outbox_outlined"
    CCM_PASTE_OUTLINED = "ccm-paste_outlined"
    CCM_REPLACE_OUTLINED = "ccm-replace_outlined"
    CCM_RICHTEXT_OUTLINED = "ccm-richtext_outlined"
    CCM_STRAIGHT_LINE_OUTLINED = "ccm-straight-line_outlined"
    CHAT_SEARCH_SOLID_OUTLINED = "chat-search-solid_outlined"
    CIRCULAREND_POINT_OUTLINED = "circularend-point_outlined"
    COLLAPSE_KEYBOARD_OUTLINED = "collapse-keyboard_outlined"
    COMMENT_NUMBER_A_OUTLINED = "comment-number-a_outlined"
    COMPANY_OUTLINED = "company_outlined"
    CONNECT_OUTLINED = "connect_outlined"
    COPY_AND_PASTE_ITEM_OUTLINED = "copy-and-paste-item_outlined"
    CROP_OUTLINED = "crop_outlined"
    DISORDE_LIST_OUTLINED = "disorde-list_outlined"
    DOC_OUTLINED = "doc_outlined"
    DOTTED_DOTTED_LINE_OUTLINED = "dotted-dotted-line_outlined"
    DOWN_EXPAND_OUTLINED = "down-expand_outlined"
    DOWNLOAD_IMAGE_OUTLINED = "download-image_outlined"
    DRAFT_OUTLINED = "draft_outlined"
    EXPAND_DOWN_OUTLINED = "expand-down_outlined"
    FILL_FRAME_OUTLINED = "fill-frame_outlined"
    FIT_TO_FRAME_OUTLINED = "fit-to-frame_outlined"
    FOLDER_ADD_OUTLINED = "folder-add_outlined"
    FOLLOW_OUTLINED = "follow_outlined"
    FORYOU_OUTLINED = "foryou_outlined"
    GIFT_BAG_LINEAR_OUTLINED = "gift-bag-linear_outlined"
    GRADUATION_OUTLINED = "graduation_outlined"
    HOT_OUTLINED = "hot_outlined"
    INSERT_COLUMNS_OUTLINED = "insert-columns_outlined"
    JOIN_MEETING_OUTLINED = "join-meeting_outlined"
    KANBAN_OUTLINED = "kanban_outlined"
    LIKE_LINE_OUTLINED = "like-line_outlined"
    LINE_AREND_POINT_OUTLINED = "line-arend-point_outlined"
    LIST_SETTING_OUTLINED = "list-setting_outlined"
    LISTVIEW_OUTLINED = "listview_outlined"
    MIC_OUTLINE_OUTLINED = "mic-outline_outlined"
    MINDNOTE_OUTLINED = "mindnote_outlined"
    MOVE_SELECTED_OUTLINED = "move-selected_outlined"
    MSGCARD_RECTANGLE_OUTLINED = "msgcard-rectangle_outlined"
    NEW_ITEM_OUTLINED = "new-item_outlined"
    ORDE_LIST_OUTLINED = "orde-list_outlined"
    ORIGINAL_SIZE_WIEW_OUTLINED = "original-size-wiew_outlined"
    PENCILKIT_OUTLINED = "pencilkit_outlined"
    PRINT_OUTLINED = "print_outlined"
    PROPORTION_OUTLINED = "proportion_outlined"
    PUSH_LEFT_OUTLINED = "push-left_outlined"
    QUICK_ACCESS_OUTLINED = "quick-access_outlined"
    ROUND_ADD_OUTLINED = "round-add_outlined"
    SENT_BACK_OUTLINED = "sent-back_outlined"
    SET_AS_TEXT_OUTLINED = "set-as-text_outlined"
    SHAPE_OUTLINED = "shape_outlined"
    SHEET_OUTLINED = "sheet_outlined"
    SHEET_DATAVERIFICATION_OUTLINED = "sheet-dataverification_outlined"
    SHORT_CUTS_OUTLINED = "short-cuts_outlined"
    SOR_A_TO_Z_OUTLINED = "sor-a-to-z_outlined"
    SORT_OUTLINED = "sort_outlined"
    SOR_Z_TO_A_OUTLINED = "sor-z-to-a_outlined"
    SQUARE_DOTTED_LINE_OUTLINED = "square-dotted-line_outlined"
    SQUAREND_POINT_OUTLINED = "squarend-point_outlined"
    START_OUTLINED = "start_outlined"
    STATUS_MEETING_OUTLINED = "status-meeting_outlined"
    STATUS_TRIP_OUTLINED = "status-trip_outlined"
    STATUS_VACATION_OUTLINED = "status-vacation_outlined"
    STRAIGHT_LINE_OUTLINED = "straight-line_outlined"
    STRIKETHROUGH_OUTLINED = "strikethrough_outlined"
    SUBTITLE_SETTING_OUTLINED = "subtitle-setting_outlined"
    TAB_FIX_OUTLINED = "tab-fix_outlined"
    TAB_RELEASE_OUTLINED = "tab-release_outlined"
    TAB_TODO_OUTLINED = "tab-todo_outlined"
    TEX_OUTLINED = "tex_outlined"
    TOOLBAR_TOGGLE_OUTLINED = "toolbar-toggle_outlined"
    TRIANGULAREND_POINT_OUTLINED = "triangularend-point_outlined"
    TRUSTPARTY_OUTLINED = "trustparty_outlined"
    VIEW_VIDEO_OUTLINED = "view-video_outlined"
    WAITING_OUTLINED = "waiting_outlined"
    WIDELIST_OUTLINED = "widelist_outlined"
    WORKORDER_OUTLINED = "workorder_outlined"
    ADD_FILLED = "add_filled"
    ALARM_CLOCK_FILLED = "alarm-clock_filled"
    ALERTS_OFF_FILLED = "alerts-off_filled"
    APP_DEFAULT_FILLED = "app-default_filled"
    BAN_FILLED = "ban_filled"
    BAN_CANCEL_FILLED = "ban-cancel_filled"
    BELL_FILLED = "bell_filled"
    CAMERA_FILLED = "camera_filled"
    CLOSE_FILLED = "close_filled"
    COLLECT_FILLED = "collect_filled"
    DELETE_FILLED = "delete_filled"
    DELETE_TRASH_FILLED = "delete-trash_filled"
    DOWN_ROUND_FILLED = "down-round_filled"
    EMPTY_FILLED = "empty_filled"
    ERROR_FILLED = "error_filled"
    ERROR_HOLLOW_FILLED = "error-hollow_filled"
    EXPAND_DOWN_FILLED = "expand-down_filled"
    EXPAND_LEFT_FILLED = "expand-left_filled"
    EXPAND_RIGHT_FILLED = "expand-right_filled"
    EXPAND_UP_FILLED = "expand-up_filled"
    GIF_FILLED = "gif_filled"
    HISTORY_SEARCH_FILLED = "history-search_filled"
    INFO_FILLED = "info_filled"
    INFO_HOLLOW_FILLED = "info-hollow_filled"
    LABEL_CUSTOM_FILLED = "label-custom_filled"
    LIKE_FILLED = "like_filled"
    LINK_COPY_FILLED = "link-copy_filled"
    LOADFAIL_FILLED = "loadfail_filled"
    MAYBE_FILLED = "maybe_filled"
    MORE_ADD_FILLED = "more-add_filled"
    MOVE_BOTTOM_FILLED = "move-bottom_filled"
    MOVE_TOP_FILLED = "move-top_filled"
    NO_FILLED = "no_filled"
    NOPICTURE_FILLED = "nopicture_filled"
    SAFE_FILLED = "safe_filled"
    SAFE_VC_FILLED = "safe-vc_filled"
    SCORE_FILLED = "score_filled"
    SEND_FILLED = "send_filled"
    SETTING_FILLED = "setting_filled"
    SETTING_INTER_FILLED = "setting-inter_filled"
    SHARE_FILLED = "share_filled"
    SUCCEED_FILLED = "succeed_filled"
    SUCCEED_HOLLOW_FILLED = "succeed-hollow_filled"
    THUMBDOWN_FILLED = "thumbdown_filled"
    THUMBSUP_FILLED = "thumbsup_filled"
    TIME_FILLED = "time_filled"
    UP_FILLED = "up_filled"
    VERIFY_FILLED = "verify_filled"
    VIEW_IN_CHAT_FILLED = "view-in-chat_filled"
    VISIBLE_FILLED = "visible_filled"
    VISIBLE_LOCK_FILLED = "visible-lock_filled"
    WARNING_FILLED = "warning_filled"
    WARNING_HOLLOW_FILLED = "warning-hollow_filled"
    YES_FILLED = "yes_filled"
    SHEET_ICONSETS_CAUTION_FILLED = "sheet-iconsets-caution_filled"
    SHEET_ICONSETS_CHECK_FILLED = "sheet-iconsets-check_filled"
    SHEET_ICONSETS_CROSS_FILLED = "sheet-iconsets-cross_filled"
    SHEET_ICONSETS_DECREASE_FILLED = "sheet-iconsets-decrease_filled"
    SHEET_ICONSETS_DIAMOND_FILLED = "sheet-iconsets-diamond_filled"
    SHEET_ICONSETS_DOWN_GREY_FILLED = "sheet-iconsets-down-grey_filled"
    SHEET_ICONSETS_GREYCIRCLE_FILLED = "sheet-iconsets-greycircle_filled"
    SHEET_ICONSETS_HEART_100_FILLED = "sheet-iconsets-heart-100_filled"
    SHEET_ICONSETS_INCREASE_FILLED = "sheet-iconsets-increase_filled"
    SHEET_ICONSETS_REDFLAG_FILLED = "sheet-iconsets-redflag_filled"
    SHEET_ICONSETS_RIGHTDOWN_GREY_FILLED = "sheet-iconsets-rightdown-grey_filled"
    SHEET_ICONSETS_RIGHT_GREY_FILLED = "sheet-iconsets-right-grey_filled"
    SHEET_ICONSETS_RIGHTUP_GREY_FILLED = "sheet-iconsets-rightup-grey_filled"
    SHEET_ICONSETS_STABLE_FILLED = "sheet-iconsets-stable_filled"
    SHEET_ICONSETS_STAR_100_FILLED = "sheet-iconsets-star-100_filled"
    SHEET_ICONSETS_THUMBSDOWN_FILLED = "sheet-iconsets-thumbsdown_filled"
    SHEET_ICONSETS_THUMBSUP_FILLED = "sheet-iconsets-thumbsup_filled"
    SHEET_ICONSETS_TRIANGLE_FILLED = "sheet-iconsets-triangle_filled"
    SHEET_ICONSETS_UP_GREY_FILLED = "sheet-iconsets-up-grey_filled"
    BUILDING_FILLED = "building_filled"
    ROOM_FILLED = "room_filled"
    ACTIVITY_FILLED = "activity_filled"
    APPROVAL_FILLED = "approval_filled"
    APPROVAL_SINGLE_FILLED = "approval-single_filled"
    CALENDAR_FILLED = "calendar_filled"
    CALENDAR_1_FILLED = "calendar-1_filled"
    CALENDAR_2_FILLED = "calendar-2_filled"
    CALENDAR_3_FILLED = "calendar-3_filled"
    CALENDAR_4_FILLED = "calendar-4_filled"
    CALENDAR_5_FILLED = "calendar-5_filled"
    CALENDAR_6_FILLED = "calendar-6_filled"
    CALENDAR_7_FILLED = "calendar-7_filled"
    CALENDAR_8_FILLED = "calendar-8_filled"
    CALENDAR_9_FILLED = "calendar-9_filled"
    CALENDAR_10_FILLED = "calendar-10_filled"
    CALENDAR_11_FILLED = "calendar-11_filled"
    CALENDAR_12_FILLED = "calendar-12_filled"
    CALENDAR_13_FILLED = "calendar-13_filled"
    CALENDAR_14_FILLED = "calendar-14_filled"
    CALENDAR_15_FILLED = "calendar-15_filled"
    CALENDAR_16_FILLED = "calendar-16_filled"
    CALENDAR_17_FILLED = "calendar-17_filled"
    CALENDAR_18_FILLED = "calendar-18_filled"
    CALENDAR_19_FILLED = "calendar-19_filled"
    CALENDAR_20_FILLED = "calendar-20_filled"
    CALENDAR_21_FILLED = "calendar-21_filled"
    CALENDAR_22_FILLED = "calendar-22_filled"
    CALENDAR_23_FILLED = "calendar-23_filled"
    CALENDAR_24_FILLED = "calendar-24_filled"
    CALENDAR_25_FILLED = "calendar-25_filled"
    CALENDAR_26_FILLED = "calendar-26_filled"
    CALENDAR_27_FILLED = "calendar-27_filled"
    CALENDAR_28_FILLED = "calendar-28_filled"
    CALENDAR_29_FILLED = "calendar-29_filled"
    CALENDAR_30_FILLED = "calendar-30_filled"
    CALENDAR_31_FILLED = "calendar-31_filled"
    CALENDAR_CHAT_FILLED = "calendar-chat_filled"
    CALENDAR_CHAT_AVATAR_FILLED = "calendar-chat-avatar_filled"
    CALENDAR_DOWN_FILLED = "calendar-down_filled"
    CALENDAR_LEFT_FILLED = "calendar-left_filled"
    CALENDAR_RIGHT_FILLED = "calendar-right_filled"
    CALENDAR_UP_FILLED = "calendar-up_filled"
    EXTERNAL_FILLED = "external_filled"
    FLAG_FILLED = "flag_filled"
    FORWARD_FILLED = "forward_filled"
    FORWARD_ALL_FILLED = "forward-all_filled"
    FORWARD_COM_FILLED = "forward-com_filled"
    GIFT_BAG_FILLED = "gift-bag_filled"
    INBOX_FILLED = "inbox_filled"
    LANGUAGE_FILLED = "language_filled"
    LATER_FILLED = "later_filled"
    MAIL_FILLED = "mail_filled"
    REPLY_FILLED = "reply_filled"
    REPLY_ALL_FILLED = "reply-all_filled"
    RESOLVE_FILLED = "resolve_filled"
    SHARELINK_FILLED = "sharelink_filled"
    TAG_FILLED = "tag_filled"
    BOARDS_FILLED = "boards_filled"
    BUZZ_FILLED = "buzz_filled"
    CHAT_FILLED = "chat_filled"
    CHAT_ALL_FILLED = "chat-all_filled"
    CHAT_DONE_FILLED = "chat-done_filled"
    CHAT_NEWS_FILLED = "chat-news_filled"
    CHAT_SECRET_FILLED = "chat-secret_filled"
    CHAT_TOPIC_FILLED = "chat-topic_filled"
    CHAT_TRANS_FILLED = "chat-trans_filled"
    LOCK_CHAT_FILLED = "lock-chat_filled"
    REPLY_CN_FILLED = "reply-cn_filled"
    SPEED_FILLED = "speed_filled"
    COMMUNITY_TAB_FILLED = "community-tab_filled"
    CONTACTS_FILLED = "contacts_filled"
    EMOJI_FILLED = "emoji_filled"
    FEMALE_FILLED = "female_filled"
    GROUP_FILLED = "group_filled"
    GROUP_EXIT_FILLED = "group-exit_filled"
    MALE_FILLED = "male_filled"
    MEMBER_FILLED = "member_filled"
    PERSON_ADD_FILLED = "person-add_filled"
    PERSON_REMOVE_FILLED = "person-remove_filled"
    ROBOT_FILLED = "robot_filled"
    SPECIAL_ATTENTION_FILLED = "special-attention_filled"
    VC_REACTION_FILLED = "vc-reaction_filled"
    ADDMEDIA_FILLED = "addmedia_filled"
    ANNOTATE_FILLED = "annotate_filled"
    ANNOUNCE_FILLED = "announce_filled"
    BLUETOOTH_FILLED = "bluetooth_filled"
    BREAKOUTROOMS_FILLED = "breakoutrooms_filled"
    CALL_FILLED = "call_filled"
    CALL_END_FILLED = "call-end_filled"
    CALL_NET_FILLED = "call-net_filled"
    CALL_OUT_FILLED = "call-out_filled"
    CALL_SECRET_FILLED = "call-secret_filled"
    CALL_VIDEO_FILLED = "call-video_filled"
    CODING_TEST_FILLED = "coding-test_filled"
    EAR_FILLED = "ear_filled"
    EFFECTS_FILLED = "effects_filled"
    GRID_VIEW_FILLED = "grid-view_filled"
    HEADPHONE_FILLED = "headphone_filled"
    HIDE_SIDEBAR_FILLED = "hide-sidebar_filled"
    HOST_CONTROL_FILLED = "host-control_filled"
    LABS_FILLED = "labs_filled"
    LEAVEROOM_FILLED = "leaveroom_filled"
    LIVESTREAM_FILLED = "livestream_filled"
    LIVESTREAM_ING_FILLED = "livestream-ing_filled"
    LIVESTREAM_START_FILLED = "livestream-start_filled"
    LOCK_FILLED = "lock_filled"
    LOCK_OPEN_FILLED = "lock-open_filled"
    LOWERHAND_FILLED = "lowerhand_filled"
    MAC_FILLED = "mac_filled"
    MAGIC_SHARE_FILLED = "magic-share_filled"
    MEETING_FILLED = "meeting_filled"
    MIC_FILLED = "mic_filled"
    MIC_OFF_FILLED = "mic-off_filled"
    MULTISTREAMING_FILLED = "multistreaming_filled"
    NEW_JOIN_MEETING_FILLED = "new-join-meeting_filled"
    NEXT_PLAY_FILLED = "next-play_filled"
    OFFICEPHONE_FILLED = "officephone_filled"
    PAUSE_FILLED = "pause_filled"
    PAUSE_ROUND_FILLED = "pause-round_filled"
    PLAY_FILLED = "play_filled"
    PLAY_ROUND_FILLED = "play-round_filled"
    PREVIOUS_PLAY_FILLED = "previous-play_filled"
    RAISEHAND_FILLED = "raisehand_filled"
    RECORD_FILLED = "record_filled"
    REJECT_FILLED = "reject_filled"
    SHARE_SCREEN_FILLED = "share-screen_filled"
    SHOW_SIDEBAR_FILLED = "show-sidebar_filled"
    SIP_FILLED = "sip_filled"
    SPEAKER_FILLED = "speaker_filled"
    SPEAKER_MUTE_FILLED = "speaker-mute_filled"
    STOP_RECORD_FILLED = "stop-record_filled"
    SUBTITLES_FILLED = "subtitles_filled"
    VC_TOOLBAR_DOWN_FILLED = "vc-toolbar-down_filled"
    VC_TOOLBAR_UP_FILLED = "vc-toolbar-up_filled"
    VIDEO_FILLED = "video_filled"
    VIDEO_OFF_FILLED = "video-off_filled"
    VIRTUAL_OFFICE_FILLED = "virtual-office_filled"
    WINS_FILLED = "wins_filled"
    APP_FILLED = "app_filled"
    CHAT_PIN_FILLED = "chat-pin_filled"
    LOCAL_FILLED = "local_filled"
    PIN_FILLED = "pin_filled"
    GADGET_FILLED = "gadget_filled"
    CELLPHONE_FILLED = "cellphone_filled"
    COMPUTER_FILLED = "computer_filled"
    MOUSE_FILLED = "mouse_filled"
    PC_FILLED = "pc_filled"
    ARRANGE_FILLED = "arrange_filled"
    COPY_FILLED = "copy_filled"
    DESCRIBE_FILLED = "describe_filled"
    FEEDBACK_FILLED = "feedback_filled"
    FOLDER_FILLED = "folder_filled"
    HELPDESK_FILLED = "helpdesk_filled"
    HOME_FILLED = "home_filled"
    MULTI_FOLDER_FILLED = "multi-folder_filled"
    MYFOLDER_FILLED = "myfolder_filled"
    NOTE_FILLED = "note_filled"
    ORGANIZATION_FILLED = "organization_filled"
    PREMIUM_GLEAM_FILLED = "premium-gleam_filled"
    SCROLL_FILLED = "scroll_filled"
    SPACE_FILLED = "space_filled"
    SPACE_MYSPACE_FILLED = "space-myspace_filled"
    SPACE_SHAREDFOLDER_FILLED = "space-sharedfolder_filled"
    TODO_FILLED = "todo_filled"
    WIKI_FILLED = "wiki_filled"
    WIKI_BOOK_FILLED = "wiki-book_filled"
    WIKI_SPACE_FILLED = "wiki-space_filled"
    WORKSPACE_FILLED = "workspace_filled"
    EXCHANGE_FILLED = "exchange_filled"
    GOOGLE_FILLED = "google_filled"
    HIRELOGO_FILLED = "hirelogo_filled"
    INTERVIEW_SPACE_FILLED = "interview-space_filled"
    LARK_LOGO_FILLED = "lark-logo_filled"
    MEEGO_FILLED = "meego_filled"
    MINUTES_LOGO_FILLED = "minutes-logo_filled"
    WECHAT_FILLED = "wechat_filled"
    WECHAT_FRIEND_FILLED = "wechat-friend_filled"
    WEIBO_FILLED = "weibo_filled"
    ATTACHMENT_FILLED = "attachment_filled"
    BUSINESS_CARD_FILLED = "business-card_filled"
    CANCEL_LINK_FILLED = "cancel-link_filled"
    CLOSE_BANNER_FILLED = "close-banner_filled"
    DANMU_OFF_FILLED = "danmu-off_filled"
    DANMU_ON_FILLED = "danmu-on_filled"
    DIAGNOSE_FILLED = "diagnose_filled"
    FILE_LINK_VIDEO_FILLED = "file-link-video_filled"
    GROUP_ADD_FILLED = "group-add_filled"
    HISTORY_FILLED = "history_filled"
    HUB_FILLED = "hub_filled"
    JOBS_FILLED = "jobs_filled"
    JOIN_MEETING_FILLED = "join-meeting_filled"
    LUCKMONEY_FILLED = "luckmoney_filled"
    PHOTO_ALBUM_FILLED = "photo-album_filled"
    REFERRALS_FILLED = "referrals_filled"
    STATUS_MEETING_FILLED = "status-meeting_filled"
    STATUS_TRIP_FILLED = "status-trip_filled"
    STATUS_VACATION_FILLED = "status-vacation_filled"
    TAB_APP_FILLED = "tab-app_filled"
    TAB_CHAT_FILLED = "tab-chat_filled"
    TAB_COLLECT_FILLED = "tab-collect_filled"
    TAB_COMMUNITY_FILLED = "tab-community_filled"
    TAB_CONTACTS_FILLED = "tab-contacts_filled"
    TAB_DRIVE_FILLED = "tab-drive_filled"
    TAB_MAIL_FILLED = "tab-mail_filled"
    TAB_MINUTES_FILLED = "tab-minutes_filled"
    TAB_PIN_FILLED = "tab-pin_filled"
    TAB_TAG_FILLED = "tab-tag_filled"
    TAB_TODO_FILLED = "tab-todo_filled"
    TAB_VIDEO_FILLED = "tab-video_filled"
    TAB_WIKI_FILLED = "tab-wiki_filled"
    VOTE_FILLED = "vote_filled"
    WORKORDER_FILLED = "workorder_filled"
    WORKORDER_CLOSE_FILLED = "workorder-close_filled"
    CALENDAR_COLORFUL = "calendar_colorful"
    TODO_COLORFUL = "todo_colorful"
    VOTE_COLORFUL = "vote_colorful"
    FILE_LARK_MINUTES_COLORFUL = "file-lark-minutes_colorful"
    FILE_FORM_COLORFUL = "file-form_colorful"
    WIKI_BITABLE_COLORFUL = "wiki-bitable_colorful"
    LARKCOMMUNITY_COLORFUL = "larkcommunity_colorful"
    HIRELOGO_COLORFUL = "hirelogo_colorful"
    LARK_LOGO_COLORFUL = "lark-logo_colorful"
    MEEGO_COLORFUL = "meego_colorful"
    MYAI_COLORFUL = "myai_colorful"
    APAAS_COLORFUL = "apaas_colorful"
    APPROVAL_COLORFUL = "approval_colorful"


class UDIconStyle(BaseModel):
    color: Color = Color.RED.value
    """图标颜色。支持设置线性和面性图标（即 token 末尾为 `outlined` 或 `filled` 的图标）的颜色。"""


class UDIconElement(BaseModel):
    token: UDIconToken
    """图标的 token"""
    style: Union[UDIconStyle, None] = None



from typing import List, Union

from typing_extensions import Literal

from ..base import BaseElement, BaseModel
from ..icon import IconElement
from ..ud_icon import UDIconElement


class PersonListPerson(BaseModel):
    user_id: str


class PersonListElement(BaseElement):
    tag: str = "person_list"
    lines: Union[int, None] = None
    show_name: bool = True
    show_avatar: bool = False
    size: Literal["extra_small", "small", "medium", "large"] = "medium"
    persons: List[PersonListPerson]
    icon: Union[IconElement, None] = None
    ud_icon: Union[UDIconElement, None] = None



import re
from typing import Union

import httpx
from pydantic import BaseModel

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.drive.file import AsyncFile, File
from slark.types._utils import cached_property


class FileInfo(BaseModel):
    file_token: str


class AsyncDrive(AsyncAPIResource):
    @cached_property
    def file(self):
        return AsyncFile(client=self._client)

    async def get_file_info(self, url: str) -> FileInfo:
        if "/file/" in url:
            file_token = re.findall(r"\/file\/([^\/\?]+)", url)[0]
        else:
            raise ValueError("The url is not a file url")
        return FileInfo(file_token=file_token)

    async def download_file(
        self,
        url: str,
        *,
        save_dir: str,
        range: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        info = await self.get_file_info(url)
        return await self.file.download(
            info.file_token, save_dir=save_dir, range=range, timeout=timeout
        )


class Drive(APIResource):
    @cached_property
    def file(self):
        return File(client=self._client)

    def get_file_info(self, url: str) -> FileInfo:
        if "/file/" in url:
            file_token = re.findall(r"\/file\/([^\/\?]+)", url)[0]
        else:
            raise ValueError("The url is not a file url")
        return FileInfo(file_token=file_token)

    def download_file(
        self,
        url: str,
        *,
        save_dir: str,
        range: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        info = self.get_file_info(url)
        return self.file.download(info.file_token, save_dir=save_dir, range=range, timeout=timeout)



from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.knowledge_space.nodes import AsyncNodes, Nodes
from slark.types._utils import cached_property


class AsyncKnowledgeSpace(AsyncAPIResource):
    @cached_property
    def nodes(self):
        return AsyncNodes(client=self._client)


class KnowledgeSpace(APIResource):
    @cached_property
    def nodes(self):
        return Nodes(client=self._client)



from typing import Union

import httpx

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.types.card.card import InteractiveCard
from slark.types.response import BaseResponse
from slark.utils import card_builder as builder
from slark.utils.time import datetime_now


class AsyncWebhook(AsyncAPIResource):
    async def send_card(
        self, card: InteractiveCard, timeout: Union[httpx.Timeout, None] = None
    ) -> BaseResponse:
        return await self._post(
            self._client._webhook_url,
            body={
                "msg_type": "interactive",
                "card": card.model_dump(),
            },
            cast_to=BaseResponse,
            options={"timeout": timeout, "no_auth": True},
        )

    async def post_error_card(
        self,
        msg: str,
        traceback: str,
        title,
        subtitle: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        if subtitle is None:
            subtitle = datetime_now()
        return await self.send_card(
            builder.build_error_msg_card(msg, traceback, title, subtitle),
            timeout=timeout,
        )

    async def post_success_card(
        self,
        msg: str,
        title,
        subtitle: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        if subtitle is None:
            subtitle = datetime_now()
        return await self.send_card(
            builder.build_success_msg_card(msg, title, subtitle),
            timeout=timeout,
        )


class Webhook(APIResource):
    def send_card(
        self, card: InteractiveCard, timeout: Union[httpx.Timeout, None] = None
    ) -> BaseResponse:
        return self._post(
            self._client._webhook_url,
            body={
                "msg_type": "interactive",
                "card": card.model_dump(),
            },
            cast_to=BaseResponse,
            options={"timeout": timeout, "no_auth": True},
        )

    def post_error_card(
        self,
        msg: str,
        traceback: str,
        title,
        subtitle: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        if subtitle is None:
            subtitle = datetime_now()
        return self.send_card(
            builder.build_error_msg_card(msg, traceback, title, subtitle),
            timeout=timeout,
        )

    async def post_success_card(
        self,
        msg: str,
        title,
        subtitle: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        if subtitle is None:
            subtitle = datetime_now()
        return self.send_card(
            builder.build_success_msg_card(msg, title, subtitle),
            timeout=timeout,
        )



from typing import List

from slark.types._common import BaseModel
from slark.types.response import BaseResponse

from .common import LarkGetMessage


class SendMessageResponse(BaseResponse):
    data: LarkGetMessage


class ReplyMessageResponse(BaseResponse):
    data: LarkGetMessage


class EditMessageResponse(BaseResponse):
    data: LarkGetMessage


class ForwardMessageResponse(BaseResponse):
    data: LarkGetMessage


class GetMessageData(BaseModel):
    items: List[LarkGetMessage]
    """消息内容。注意：如果查询的消息类型为合并转发（merge_forward），则返回的 items 中会包含 1 条合并转发消息和 N 条子消息。"""


class GetMessageResponse(BaseResponse):
    data: GetMessageData



from .response import (
    GetAllWorksheetsInfoResponse,
    GetSpreadsheetInfoResponse,
    GetWorksheetInfoResponse,
)

__all__ = [
    "GetSpreadsheetInfoResponse",
    "GetAllWorksheetsInfoResponse",
    "GetWorksheetInfoResponse",
]



from typing import TYPE_CHECKING, Any, Callable, Generic, TypeVar, Union, overload

from typing_extensions import Self

_T = TypeVar("_T")
# cached properties
if TYPE_CHECKING:
    cached_property = property

    # we define a separate type (copied from typeshed)
    # that represents that `cached_property` is `set`able
    # at runtime, which differs from `@property`.
    #
    # this is a separate type as editors likely special case
    # `@property` and we don't want to cause issues just to have
    # more helpful internal types.

    class typed_cached_property(Generic[_T]):
        func: Callable[[Any], _T]
        attrname: Union[str, None]

        def __init__(self, func: Callable[[Any], _T]) -> None: ...

        @overload
        def __get__(self, instance: None, owner: type[Any] | None = None) -> Self: ...

        @overload
        def __get__(self, instance: object, owner: type[Any] | None = None) -> _T: ...

        def __get__(self, instance: object, owner: type[Any] | None = None) -> _T | Self:
            raise NotImplementedError()

        def __set_name__(self, owner: type[Any], name: str) -> None: ...

        # __set__ is not defined at runtime, but @cached_property is designed to be settable
        def __set__(self, instance: object, value: _T) -> None: ...
else:
    try:
        from functools import cached_property as cached_property
    except ImportError:
        from cached_property import cached_property as cached_property

    typed_cached_property = cached_property



import re
from typing import List, Union

import httpx
import pandas as pd
from loguru import logger
from typing_extensions import Literal

from slark._constants import DEFAULT_WRITE_COL_BATCH_SIZE, DEFAULT_WRITE_ROW_BATCH_SIZE
from slark.resources._resources import APIResource, AsyncAPIResource
from slark.types._common import BaseModel
from slark.types._utils import cached_property
from slark.types.spreadsheets.data import CellTypes, Range
from slark.utils import dataframe_to_values, values_to_dataframe

from .data import AsyncData, Data
from .table import AsyncTable, Table
from .worksheet import AsyncWorksheet, Worksheet


class SheetInfo(BaseModel):
    spreadsheet_token: str
    sheet_id: str
    sheet_range: Range


class AsyncSpreadsheets(AsyncAPIResource):
    @cached_property
    def table(self):
        return AsyncTable(client=self._client)

    @cached_property
    def worksheet(self):
        return AsyncWorksheet(client=self._client)

    @cached_property
    def data(self):
        return AsyncData(client=self._client)

    async def get_sheet_info(self, url: str) -> SheetInfo:
        if "/sheets/" in url:
            spreadsheet_token, sheet_id = re.findall(
                r"\/sheets\/([^\/\?]+)(?:\?sheet=([^\/]+))?", url
            )[0]
        elif "/wiki/" in url:
            wiki_token, sheet_id = re.findall(r"\/wiki\/([^\/\?]+)(?:\?sheet=([^\/]+))?", url)[0]
            node_info = (
                await self._client.knowledge_space.nodes.get_node_info(wiki_token)
            ).data.node
            if node_info.obj_type != "sheet":
                raise ValueError("The node is not a sheet")
            spreadsheet_token = node_info.obj_token
        assert spreadsheet_token, "Sheet token is not found"
        if not sheet_id:
            info = (await self.worksheet.get_all_worksheets_info(spreadsheet_token)).data.sheets[0]
            sheet_id = info.sheet_id

        else:
            info = (
                await self.worksheet.get_worksheet_info(spreadsheet_token, sheet_id=sheet_id)
            ).data.sheet
        sheet_range = Range(
            rows=info.grid_properties.row_count,
            cols=info.grid_properties.column_count,
            sheet_id=sheet_id,
        )
        return SheetInfo(
            spreadsheet_token=spreadsheet_token,
            sheet_id=sheet_id,
            sheet_range=sheet_range,
        )

    async def read(
        self,
        url: str,
        *,
        start_row: int = 0,
        start_col: int = 0,
        rows: Union[int, None] = None,
        cols: Union[int, None] = None,
        has_header: bool = True,
        dropna: bool = True,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        return_raw: bool = False,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> Union[pd.DataFrame, List[List[CellTypes]]]:
        """从电子表格读取数据，该接口返回数据的最大限制为 10 MB。该接口不支持获取跨表引用和数组公式的计算结果。

        Args:
            url (str): 电子表格的 URL 链接
            start_row (int, optional): 起始行数. Defaults to 0.
            start_col (int, optional): 起始列数. Defaults to 0.
            rows (Union[int, None], optional): 读取行数. Defaults to None.
            cols (Union[int, None], optional): 读取列数. Defaults to None.
            has_header (bool, optional): 是否包括标题. Defaults to True.
            dropna (bool, optional): 是否去除全为空的行/列. Defaults to True.
            valueRenderOption (Literal[ &quot;ToString&quot;, &quot;Formula&quot;, &quot;FormattedValue&quot;, &quot;UnformattedValue&quot; ], None, optional): \
                指定单元格数据的格式。可选值如下所示。\
                当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。\
                ToString：返回纯文本的值（数值类型除外）。\
                Formula：单元格中含有公式时，返回公式本身。\
                FormattedValue：计算并格式化单元格。\
                UnformattedValue：计算但不对单元格进行格式化. Defaults to None.
            dateTimeRenderOption (Literal[&quot;FormattedString&quot;], None, optional):\
                指定数据类型为日期、时间、或时间日期的单元格数据的格式。\
                若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；\
                小数部分为该时间占 24 小时的份额。\
                例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。\
                其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；\
                0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。\
                可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。. Defaults to None.
            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;], None, optional):\
                当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。\
                默认为 lark_id，建议选择 open_id 或 union_id. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            Union[pd.DataFrame, List[List[CellTypes]]]: 读取的数据，\
                如果 return_raw 为 True 则返回 List[List[CellTypes]]，否则返回 pd.DataFrame
        """
        info = await self.get_sheet_info(url)
        read_range = info.sheet_range.intersect(
            Range(
                start_row=start_row,
                start_col=start_col,
                rows=rows or info.sheet_range.rows,
                cols=cols or info.sheet_range.cols,
                sheet_id=info.sheet_id,
            )
        )
        data = await self.data.read_single_range_data(
            info.spreadsheet_token,
            range=read_range.excel,
            valueRenderOption=valueRenderOption,
            dateTimeRenderOption=dateTimeRenderOption,
            user_id_type=user_id_type,
            timeout=timeout,
        )
        if return_raw:
            return data.data.valueRange.values
        else:
            return values_to_dataframe(
                data.data.valueRange.values, has_header=has_header, dropna=dropna
            )

    async def _write_batch(
        self,
        spreadsheet_token: str,
        write_range: Range,
        values: List[List[CellTypes]],
        mode: Literal["overwrite", "append", "prepend"],
        row_batch_size: int = DEFAULT_WRITE_ROW_BATCH_SIZE,
        col_batch_size: int = DEFAULT_WRITE_COL_BATCH_SIZE,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        def batch_range_values():
            for start_row in range(0, write_range.rows, row_batch_size):
                for start_col in range(0, write_range.cols, col_batch_size):
                    rows = min(row_batch_size, write_range.rows - start_row)
                    cols = min(col_batch_size, write_range.cols - start_col)
                    yield (
                        Range(
                            start_row=write_range.start_row + start_row,
                            start_col=write_range.start_col + start_col,
                            rows=rows,
                            cols=cols,
                            sheet_id=write_range.sheet_id,
                        ),
                        [
                            row[start_col : start_col + cols]
                            for row in values[start_row : start_row + rows]
                        ],
                    )

        for range_, values_ in batch_range_values():
            write_kwargs = {
                "spreadsheet_token": spreadsheet_token,
                "range": range_.excel,
                "values": values_,
            }
            logger.debug(f"Writing data with mode {mode}, range {range_.excel}")
            if mode == "overwrite":
                res = await self.data.write_single_range_data(**write_kwargs)
            elif mode == "append":
                res = await self.data.append_data(**write_kwargs, insertDataOption=insertDataOption)
            elif mode == "prepend":
                res = await self.data.prepend_data(**write_kwargs)
            else:
                raise ValueError(f"Invalid mode {mode}")
        return res

    async def _write(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        has_header: bool = True,
        mode: Literal["overwrite", "append", "prepend"] = "overwrite",
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        write_cols = data.shape[1]
        values = dataframe_to_values(data, has_header=has_header)
        write_rows = len(values)

        info = await self.get_sheet_info(url)
        write_range = Range(
            start_row=start_row,
            start_col=start_col,
            rows=write_rows,
            cols=write_cols,
            sheet_id=info.sheet_id,
        )

        logger.debug(f"Writing data to {url} with mode {mode}, range {write_range}")
        return await self._write_batch(
            info.spreadsheet_token,
            write_range,
            values,
            mode=mode,
            insertDataOption=insertDataOption,
        )

    async def write(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        has_header: bool = True,
    ):
        return await self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=has_header,
            mode="overwrite",
        )

    async def append(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        return await self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=False,
            mode="append",
            insertDataOption=insertDataOption,
        )

    async def prepend(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
    ):
        return await self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=False,
            mode="prepend",
        )


class Spreadsheets(APIResource):
    @cached_property
    def table(self):
        return Table(client=self._client)

    @cached_property
    def worksheet(self):
        return Worksheet(client=self._client)

    @cached_property
    def data(self):
        return Data(client=self._client)

    def get_sheet_info(self, url: str) -> SheetInfo:
        if "/sheets/" in url:
            spreadsheet_token, sheet_id = re.findall(
                r"\/sheets\/([^\/\?]+)(?:\?sheet=([^\/]+))?", url
            )[0]
        elif "/wiki/" in url:
            wiki_token, sheet_id = re.findall(r"\/wiki\/([^\/\?]+)(?:\?sheet=([^\/]+))?", url)[0]
            node_info = (self._client.knowledge_space.nodes.get_node_info(wiki_token)).data.node
            if node_info.obj_type != "sheet":
                raise ValueError("The node is not a sheet")
            spreadsheet_token = node_info.obj_token
        assert spreadsheet_token, "Sheet token is not found"
        if not sheet_id:
            info = (self.worksheet.get_all_worksheets_info(spreadsheet_token)).data.sheets[0]
            sheet_id = info.sheet_id

        else:
            info = (
                self.worksheet.get_worksheet_info(spreadsheet_token, sheet_id=sheet_id)
            ).data.sheet
        sheet_range = Range(
            rows=info.grid_properties.row_count,
            cols=info.grid_properties.column_count,
            sheet_id=sheet_id,
        )
        return SheetInfo(
            spreadsheet_token=spreadsheet_token,
            sheet_id=sheet_id,
            sheet_range=sheet_range,
        )

    def read(
        self,
        url: str,
        *,
        start_row: int = 0,
        start_col: int = 0,
        rows: Union[int, None] = None,
        cols: Union[int, None] = None,
        has_header: bool = True,
        dropna: bool = True,
        valueRenderOption: Union[
            Literal["ToString", "Formula", "FormattedValue", "UnformattedValue"], None
        ] = None,
        dateTimeRenderOption: Union[Literal["FormattedString"], None] = None,
        user_id_type: Union[Literal["open_id", "union_id"], None] = None,
        return_raw: bool = False,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> Union[pd.DataFrame, List[List[CellTypes]]]:
        """从电子表格读取数据，该接口返回数据的最大限制为 10 MB。该接口不支持获取跨表引用和数组公式的计算结果。

        Args:
            url (str): 电子表格的 URL 链接
            start_row (int, optional): 起始行数. Defaults to 0.
            start_col (int, optional): 起始列数. Defaults to 0.
            rows (Union[int, None], optional): 读取行数. Defaults to None.
            cols (Union[int, None], optional): 读取列数. Defaults to None.
            has_header (bool, optional): 是否包括标题. Defaults to True.
            dropna (bool, optional): 是否去除全为空的行/列. Defaults to True.
            valueRenderOption (Literal[ &quot;ToString&quot;, &quot;Formula&quot;, &quot;FormattedValue&quot;, &quot;UnformattedValue&quot; ], None, optional): \
                指定单元格数据的格式。可选值如下所示。\
                当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。\
                ToString：返回纯文本的值（数值类型除外）。\
                Formula：单元格中含有公式时，返回公式本身。\
                FormattedValue：计算并格式化单元格。\
                UnformattedValue：计算但不对单元格进行格式化. Defaults to None.
            dateTimeRenderOption (Literal[&quot;FormattedString&quot;], None, optional):\
                指定数据类型为日期、时间、或时间日期的单元格数据的格式。\
                若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；\
                小数部分为该时间占 24 小时的份额。\
                例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。\
                其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；\
                0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。\
                可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。. Defaults to None.
            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;], None, optional):\
                当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。\
                默认为 lark_id，建议选择 open_id 或 union_id. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            Union[pd.DataFrame, List[List[CellTypes]]]: 读取的数据，\
                如果 return_raw 为 True 则返回 List[List[CellTypes]]，否则返回 pd.DataFrame
        """
        info = self.get_sheet_info(url)
        read_range = info.sheet_range.intersect(
            Range(
                start_row=start_row,
                start_col=start_col,
                rows=rows or info.sheet_range.rows,
                cols=cols or info.sheet_range.cols,
                sheet_id=info.sheet_id,
            )
        )
        data = self.data.read_single_range_data(
            info.spreadsheet_token,
            range=read_range.excel,
            valueRenderOption=valueRenderOption,
            dateTimeRenderOption=dateTimeRenderOption,
            user_id_type=user_id_type,
            timeout=timeout,
        )
        if return_raw:
            return data.data.valueRange.values
        else:
            return values_to_dataframe(
                data.data.valueRange.values, has_header=has_header, dropna=dropna
            )

    def _write_batch(
        self,
        spreadsheet_token: str,
        write_range: Range,
        values: List[List[CellTypes]],
        mode: Literal["overwrite", "append", "prepend"],
        row_batch_size: int = DEFAULT_WRITE_ROW_BATCH_SIZE,
        col_batch_size: int = DEFAULT_WRITE_COL_BATCH_SIZE,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        def batch_range_values():
            for start_row in range(0, write_range.rows, row_batch_size):
                for start_col in range(0, write_range.cols, col_batch_size):
                    rows = min(row_batch_size, write_range.rows - start_row)
                    cols = min(col_batch_size, write_range.cols - start_col)
                    yield (
                        Range(
                            start_row=write_range.start_row + start_row,
                            start_col=write_range.start_col + start_col,
                            rows=rows,
                            cols=cols,
                            sheet_id=write_range.sheet_id,
                        ),
                        [
                            row[start_col : start_col + cols]
                            for row in values[start_row : start_row + rows]
                        ],
                    )

        for range_, values_ in batch_range_values():
            write_kwargs = {
                "spreadsheet_token": spreadsheet_token,
                "range": range_.excel,
                "values": values_,
            }
            logger.debug(f"Writing data with mode {mode}, range {range_.excel}")
            if mode == "overwrite":
                res = self.data.write_single_range_data(**write_kwargs)
            elif mode == "append":
                res = self.data.append_data(**write_kwargs, insertDataOption=insertDataOption)
            elif mode == "prepend":
                res = self.data.prepend_data(**write_kwargs)
            else:
                raise ValueError(f"Invalid mode {mode}")
        return res

    def _write(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        has_header: bool = True,
        mode: Literal["overwrite", "append", "prepend"] = "overwrite",
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        write_cols = data.shape[1]
        values = dataframe_to_values(data, has_header=has_header)
        write_rows = len(values)

        info = self.get_sheet_info(url)
        write_range = Range(
            start_row=start_row,
            start_col=start_col,
            rows=write_rows,
            cols=write_cols,
            sheet_id=info.sheet_id,
        )

        logger.debug(f"Writing data to {url} with mode {mode}, range {write_range}")
        return self._write_batch(
            info.spreadsheet_token,
            write_range,
            values,
            mode=mode,
            insertDataOption=insertDataOption,
        )

    def write(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        has_header: bool = True,
    ):
        return self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=has_header,
            mode="overwrite",
        )

    def append(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
        insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE",
    ):
        return self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=False,
            mode="append",
            insertDataOption=insertDataOption,
        )

    def prepend(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        start_row: int = 0,
        start_col: int = 0,
    ):
        return self._write(
            url,
            data=data,
            start_row=start_row,
            start_col=start_col,
            has_header=False,
            mode="prepend",
        )
